@@ -6,42 +6,39 @@
 --
 -----------------------------------------------------------------------------
 
-{-# OPTIONS -fno-warn-tabs #-}
--- The above warning supression flag is a temporary kludge.
--- While working on this module you are encouraged to remove it and
--- detab the module (please do the detabbing in a separate patch). See
---     http://hackage.haskell.org/trac/ghc/wiki/Commentary/CodingStyle#TabsvsSpaces
--- for details
-
 module OldCmm (
         CmmGroup, GenCmmGroup, RawCmmGroup, CmmDecl, RawCmmDecl,
         ListGraph(..),
-        CmmInfo(..), UpdateFrame(..), CmmInfoTable(..), ClosureTypeInfo(..),
+        CmmInfoTable(..), ClosureTypeInfo(..),
         CmmStatic(..), CmmStatics(..), CmmFormal, CmmActual,
+
         cmmMapGraph, cmmTopMapGraph,
+
         GenBasicBlock(..), CmmBasicBlock, blockId, blockStmts, mapBlockStmts,
+
         CmmStmt(..), CmmReturnInfo(..), CmmHinted(..),
         HintedCmmFormal, HintedCmmActual,
+
         CmmSafety(..), CmmCallTarget(..),
-        New.GenCmmDecl(..),
-        New.ForeignHint(..),
+        New.GenCmmDecl(..), New.ForeignHint(..),
+
         module CmmExpr,
-        Section(..),
-        ProfilingInfo(..), C_SRT(..)
-  ) where
+
+        Section(..), ProfilingInfo(..), C_SRT(..)
+    ) where
 
 #include "HsVersions.h"
 
 import qualified Cmm as New
-import Cmm           ( CmmInfoTable(..), GenCmmGroup, CmmStatics(..), GenCmmDecl(..),
-                       CmmFormal, CmmActual, Section(..), CmmStatic(..),
-                       ProfilingInfo(..), ClosureTypeInfo(..) )
+import Cmm ( CmmInfoTable(..), GenCmmGroup, CmmStatics(..), GenCmmDecl(..),
+             CmmFormal, CmmActual, Section(..), CmmStatic(..),
+             ProfilingInfo(..), ClosureTypeInfo(..) )
 
 import BlockId
-import CmmExpr
-import ForeignCall
 import ClosureInfo
+import CmmExpr
 import FastString
+import ForeignCall
 
 
 -- A [[BlockId]] is a local label.
@@ -49,24 +46,6 @@ import FastString
 -- just a single top-level item, because local labels map one-to-one
 -- with assembly-language labels.
 
------------------------------------------------------------------------------
---     Info Tables
------------------------------------------------------------------------------
-
-data CmmInfo
-  = CmmInfo
-      (Maybe BlockId)     -- GC target. Nothing <=> CPS won't do stack check
-                          -- JD: NOT USED BY NEW CODE GEN
-      (Maybe UpdateFrame) -- Update frame
-      CmmInfoTable        -- Info table
-
--- | A frame that is to be pushed before entry to the function.
--- Used to handle 'update' frames.
-data UpdateFrame =
-    UpdateFrame
-      CmmExpr    -- Frame header.  Behaves like the target of a 'jump'.
-      [CmmExpr]  -- Frame remainder.  Behaves like the arguments of a 'jump'.
-
 -----------------------------------------------------------------------------
 --  Cmm, CmmDecl, CmmBasicBlock
 -----------------------------------------------------------------------------
@@ -75,18 +54,19 @@ data UpdateFrame =
 -- re-orderd during code generation.
 
 -- | A control-flow graph represented as a list of extended basic blocks.
+--
+-- Code, may be empty.  The first block is the entry point.  The
+-- order is otherwise initially unimportant, but at some point the
+-- code gen will fix the order.
+--
+-- BlockIds must be unique across an entire compilation unit, since
+-- they are translated to assembly-language labels, which scope
+-- across a whole compilation unit.
 newtype ListGraph i = ListGraph [GenBasicBlock i]
-   -- ^ Code, may be empty.  The first block is the entry point.  The
-   -- order is otherwise initially unimportant, but at some point the
-   -- code gen will fix the order.
-
-   -- BlockIds must be unique across an entire compilation unit, since
-   -- they are translated to assembly-language labels, which scope
-   -- across a whole compilation unit.
 
 -- | Cmm with the info table as a data type
-type CmmGroup = GenCmmGroup CmmStatics CmmInfo (ListGraph CmmStmt)
-type CmmDecl = GenCmmDecl CmmStatics CmmInfo (ListGraph CmmStmt)
+type CmmGroup = GenCmmGroup CmmStatics CmmInfoTable (ListGraph CmmStmt)
+type CmmDecl = GenCmmDecl CmmStatics CmmInfoTable (ListGraph CmmStmt)
 
 -- | Cmm with the info tables converted to a list of 'CmmStatic' along with the info
 -- table label. If we are building without tables-next-to-code there will be no statics
@@ -108,84 +88,104 @@ type CmmBasicBlock   = GenBasicBlock CmmStmt
 instance UserOfLocalRegs i => UserOfLocalRegs (GenBasicBlock i) where
     foldRegsUsed f set (BasicBlock _ l) = foldRegsUsed f set l
 
-blockId :: GenBasicBlock i -> BlockId
--- The branch block id is that of the first block in 
+-- | The branch block id is that of the first block in
 -- the branch, which is that branch's entry point
+blockId :: GenBasicBlock i -> BlockId
 blockId (BasicBlock blk_id _ ) = blk_id
 
 blockStmts :: GenBasicBlock i -> [i]
 blockStmts (BasicBlock _ stmts) = stmts
 
-
 mapBlockStmts :: (i -> i') -> GenBasicBlock i -> GenBasicBlock i'
 mapBlockStmts f (BasicBlock id bs) = BasicBlock id (map f bs)
+
 ----------------------------------------------------------------
 --   graph maps
 ----------------------------------------------------------------
 
 cmmMapGraph    :: (g -> g') -> GenCmmGroup d h g -> GenCmmGroup d h g'
-cmmTopMapGraph :: (g -> g') -> GenCmmDecl d h g -> GenCmmDecl d h g'
-
 cmmMapGraph f tops = map (cmmTopMapGraph f) tops
+
+cmmTopMapGraph :: (g -> g') -> GenCmmDecl d h g -> GenCmmDecl d h g'
 cmmTopMapGraph f (CmmProc h l g) = CmmProc h l (f g)
 cmmTopMapGraph _ (CmmData s ds)  = CmmData s ds
 
-data CmmReturnInfo = CmmMayReturn
-                   | CmmNeverReturns
-    deriving ( Eq )
+data CmmReturnInfo
+  = CmmMayReturn
+  | CmmNeverReturns
+  deriving ( Eq )
 
 -----------------------------------------------------------------------------
---		CmmStmt
+--              CmmStmt
 -- A "statement".  Note that all branches are explicit: there are no
 -- control transfers to computed addresses, except when transfering
 -- control to a new function.
 -----------------------------------------------------------------------------
 
-data CmmStmt	-- Old-style
+data CmmStmt
   = CmmNop
   | CmmComment FastString
 
-  | CmmAssign CmmReg CmmExpr	 -- Assign to register
+  | CmmAssign CmmReg CmmExpr      -- Assign to register
 
-  | CmmStore CmmExpr CmmExpr     -- Assign to memory location.  Size is
-                                 -- given by cmmExprType of the rhs.
+  | CmmStore CmmExpr CmmExpr      -- Assign to memory location. Size is
+                                  -- given by cmmExprType of the rhs.
 
-  | CmmCall	 		 -- A call (foreign, native or primitive), with 
-     CmmCallTarget
-     [HintedCmmFormal]		 -- zero or more results
-     [HintedCmmActual]		 -- zero or more arguments
-     CmmReturnInfo
-  -- Some care is necessary when handling the arguments of these, see
-  -- [Register parameter passing] and the hack in cmm/CmmOpt.hs
+  | CmmCall                       -- A call (foreign, native or primitive), with
+      CmmCallTarget
+      [HintedCmmFormal]            -- zero or more results
+      [HintedCmmActual]            -- zero or more arguments
+      CmmReturnInfo
+      -- Some care is necessary when handling the arguments of these, see
+      -- [Register parameter passing] and the hack in cmm/CmmOpt.hs
 
   | CmmBranch BlockId             -- branch to another BB in this fn
 
   | CmmCondBranch CmmExpr BlockId -- conditional branch
 
-  | CmmSwitch CmmExpr [Maybe BlockId]   -- Table branch
-	-- The scrutinee is zero-based; 
-	--	zero -> first block
-	--	one  -> second block etc
-	-- Undefined outside range, and when there's a Nothing
+  | CmmSwitch                     -- Table branch
+      CmmExpr                       -- The scrutinee is zero-based;
+      [Maybe BlockId]               --      zero -> first block
+                                    --      one  -> second block etc
+                                    -- Undefined outside range, and when
+                                    -- there's a Nothing
 
-  | CmmJump CmmExpr      -- Jump to another C-- function,
-      [HintedCmmActual]        -- with these parameters.  (parameters never used)
+  | CmmJump                       -- Jump to another C-- function,
+      CmmExpr                       -- Target
+      (Maybe [GlobalReg])           -- Live registers at call site;
+                                    --      Nothing -> no information, assume
+                                    --                 all live
+                                    --      Just .. -> info on liveness, []
+                                    --                 means no live registers
+                                    -- This isn't all 'live' registers, just
+                                    -- the argument STG registers that are live
+                                    -- AND also possibly mapped to machine
+                                    -- registers. (So Sp, Hp, HpLim... ect
+                                    -- are never included here as they are
+                                    -- always live, only R2.., D1.. are
+                                    -- on this list)
 
-  | CmmReturn            -- Return from a native C-- function,
-      [HintedCmmActual]        -- with these return values. (parameters never used)
+  | CmmReturn                     -- Return from a native C-- function,
 
-data CmmHinted a = CmmHinted { hintlessCmm :: a, cmmHint :: New.ForeignHint }
-	   	 deriving( Eq )
+data CmmHinted a
+  = CmmHinted {
+        hintlessCmm :: a,
+        cmmHint :: New.ForeignHint
+    }
+  deriving( Eq )
 
-type HintedCmmFormal  = CmmHinted CmmFormal
-type HintedCmmActual  = CmmHinted CmmActual
+type HintedCmmFormal = CmmHinted CmmFormal
+type HintedCmmActual = CmmHinted CmmActual
 
-data CmmSafety      = CmmUnsafe | CmmSafe C_SRT | CmmInterruptible
+data CmmSafety
+  = CmmUnsafe
+  | CmmSafe C_SRT
+  | CmmInterruptible
 
 -- | enable us to fold used registers over '[CmmActual]' and '[CmmFormal]'
 instance UserOfLocalRegs CmmStmt where
   foldRegsUsed f (set::b) s = stmt s set
-    where 
+    where
       stmt :: CmmStmt -> b -> b
       stmt (CmmNop)                  = id
       stmt (CmmComment {})           = id
@@ -195,28 +195,21 @@ instance UserOfLocalRegs CmmStmt where
       stmt (CmmBranch _)             = id
       stmt (CmmCondBranch e _)       = gen e
       stmt (CmmSwitch e _)           = gen e
-      stmt (CmmJump e es)            = gen e . gen es
-      stmt (CmmReturn es)            = gen es
+      stmt (CmmJump e _)             = gen e
+      stmt (CmmReturn)               = id
 
       gen :: UserOfLocalRegs a => a -> b -> b
       gen a set = foldRegsUsed f set a
 
 instance UserOfLocalRegs CmmCallTarget where
-    foldRegsUsed f set (CmmCallee e _) = foldRegsUsed f set e
-    foldRegsUsed _ set (CmmPrim {})    = set
-
-instance UserOfSlots CmmCallTarget where
-    foldSlotsUsed f set (CmmCallee e _) = foldSlotsUsed f set e
-    foldSlotsUsed _ set (CmmPrim {})    = set
+    foldRegsUsed f set (CmmCallee e _)    = foldRegsUsed f set e
+    foldRegsUsed f set (CmmPrim _ mStmts) = foldRegsUsed f set mStmts
 
 instance UserOfLocalRegs a => UserOfLocalRegs (CmmHinted a) where
-  foldRegsUsed f set a = foldRegsUsed f set (hintlessCmm a)
-
-instance UserOfSlots a => UserOfSlots (CmmHinted a) where
-  foldSlotsUsed f set a = foldSlotsUsed f set (hintlessCmm a)
+    foldRegsUsed f set a = foldRegsUsed f set (hintlessCmm a)
 
 instance DefinerOfLocalRegs a => DefinerOfLocalRegs (CmmHinted a) where
-  foldRegsDefd f set a = foldRegsDefd f set (hintlessCmm a)
+    foldRegsDefd f set a = foldRegsDefd f set (hintlessCmm a)
 
 {-
 Discussion
@@ -232,7 +225,7 @@ conditional jump are explicit. ---NR]
 
 One possible way to fix this would be:
 
-data CmmStat = 
+data CmmStat =
   ...
   | CmmJump CmmBranchDest
   | CmmCondJump CmmExpr CmmBranchDest
@@ -259,18 +252,22 @@ So we'll stick with the way it is, and add the optimisation to the NCG.
 -}
 
 -----------------------------------------------------------------------------
---		CmmCallTarget
+--              CmmCallTarget
 --
 -- The target of a CmmCall.
 -----------------------------------------------------------------------------
 
 data CmmCallTarget
-  = CmmCallee		-- Call a function (foreign or native)
-	CmmExpr 		-- literal label <=> static call
-				-- other expression <=> dynamic call
-	CCallConv		-- The calling convention
+  = CmmCallee           -- Call a function (foreign or native)
+        CmmExpr                 -- literal label <=> static call
+                                -- other expression <=> dynamic call
+        CCallConv               -- The calling convention
+
+  | CmmPrim             -- Call a "primitive" (eg. sin, cos)
+        CallishMachOp           -- These might be implemented as inline
+                                -- code by the backend.
+        -- If we don't know how to implement the
+        -- mach op, then we can replace it with
+        -- this list of statements:
+        (Maybe [CmmStmt])
 
-  | CmmPrim		-- Call a "primitive" (eg. sin, cos)
-	CallishMachOp		-- These might be implemented as inline
-				-- code by the backend.
-  deriving Eq
