@@ -19,11 +19,11 @@ module BinIface (
 #include "HsVersions.h"
 
 import TcRnMonad
-import TyCon      (TyCon, tyConName, tupleTyConSort, tupleTyConArity, isTupleTyCon, tyConIP_maybe)
+import TyCon      (TyCon, tyConName, tupleTyConSort, tupleTyConArity, isTupleTyCon)
 import DataCon    (dataConName, dataConWorkId, dataConTyCon)
-import IParam     (ipFastString, ipTyConName)
 import PrelInfo   (wiredInThings, basicKnownKeyNames)
 import Id         (idName, isDataConWorkId_maybe)
+import CoreSyn    (DFunArg(..))
 import TysWiredIn
 import IfaceEnv
 import HscTypes
@@ -39,7 +39,6 @@ import DynFlags
 import UniqFM
 import UniqSupply
 import CostCentre
-import StaticFlags
 import Panic
 import Binary
 import SrcLoc
@@ -51,6 +50,7 @@ import Outputable
 import Platform
 import FastString
 import Constants
+import Util
 
 import Data.Bits
 import Data.Char
@@ -59,7 +59,6 @@ import Data.Word
 import Data.Array
 import Data.IORef
 import Control.Monad
-import System.Time ( ClockTime(..) )
 
 
 -- ---------------------------------------------------------------------------
@@ -77,7 +76,7 @@ readBinIface :: CheckHiWay -> TraceBinIFaceReading -> FilePath
              -> TcRnIf a b ModIface
 readBinIface checkHiWay traceBinIFaceReading hi_path = do
     ncu <- mkNameCacheUpdater
-    dflags <- getDOpts
+    dflags <- getDynFlags
     liftIO $ readBinIface_ dflags checkHiWay traceBinIFaceReading hi_path ncu
 
 readBinIface_ :: DynFlags -> CheckHiWay -> TraceBinIFaceReading -> FilePath
@@ -86,7 +85,7 @@ readBinIface_ :: DynFlags -> CheckHiWay -> TraceBinIFaceReading -> FilePath
 readBinIface_ dflags checkHiWay traceBinIFaceReading hi_path ncu = do
     let printer :: SDoc -> IO ()
         printer = case traceBinIFaceReading of
-                      TraceBinIFaceReading -> \sd -> printSDoc sd defaultDumpStyle
+                      TraceBinIFaceReading -> \sd -> log_action dflags dflags SevOutput noSrcSpan defaultDumpStyle sd
                       QuietBinIFaceReading -> \_ -> return ()
         wantedGot :: Outputable a => String -> a -> a -> IO ()
         wantedGot what wanted got =
@@ -124,7 +123,7 @@ readBinIface_ dflags checkHiWay traceBinIFaceReading hi_path ncu = do
 
     -- Check the interface file version and ways.
     check_ver  <- get bh
-    let our_ver = show opt_HiVersion
+    let our_ver = show hiVersion
     wantedGot "Version" our_ver check_ver
     errorOnMismatch "mismatched interface file versions" our_ver check_ver
 
@@ -174,7 +173,7 @@ writeBinIface dflags hi_path mod_iface = do
         else Binary.put_ bh (0 :: Word64)
 
     -- The version and way descriptor go next
-    put_ bh (show opt_HiVersion)
+    put_ bh (show hiVersion)
     let way_descr = getWayDescr dflags
     put_  bh way_descr
 
@@ -316,7 +315,7 @@ knownKeyNamesMap = listToUFM_Directly [(nameUnique n, n) | n <- knownKeyNames]
 
 -- See Note [Symbol table representation of names]
 putName :: BinDictionary -> BinSymbolTable -> BinHandle -> Name -> IO ()
-putName dict BinSymbolTable{ 
+putName _dict BinSymbolTable{ 
                bin_symtab_map = symtab_map_ref,
                bin_symtab_next = symtab_next }    bh name
   | name `elemUFM` knownKeyNamesMap
@@ -327,10 +326,6 @@ putName dict BinSymbolTable{
   = case wiredInNameTyThing_maybe name of
      Just (ATyCon tc)
        | isTupleTyCon tc             -> putTupleName_ bh tc 0
-       | Just ip <- tyConIP_maybe tc -> do
-         off <- allocateFastString dict (ipFastString ip)
-         -- MASSERT(off < 2^(30 :: Int))
-         put_ bh (0xC0000000 .|. off)
      Just (ADataCon dc)
        | let tc = dataConTyCon dc, isTupleTyCon tc -> putTupleName_ bh tc 1
      Just (AnId x)
@@ -362,7 +357,7 @@ putTupleName_ bh tc thing_tag
 getSymtabName :: NameCacheUpdater
               -> Dictionary -> SymbolTable
               -> BinHandle -> IO Name
-getSymtabName ncu dict symtab bh = do
+getSymtabName _ncu _dict symtab bh = do
     i <- get bh
     case i .&. 0xC0000000 of
         0x00000000 -> return $! symtab ! fromIntegral (i ::  Word32)
@@ -385,7 +380,6 @@ getSymtabName ncu dict symtab bh = do
                      _ -> pprPanic "getSymtabName:unknown tuple sort" (ppr i)
             thing_tag = (i .&. 0x0CFFFFFF) `shiftR` 26
             arity = fromIntegral (i .&. 0x03FFFFFF)
-        0xC0000000 -> liftM ipTyConName $ updateNameCache ncu $ flip allocateIPName (dict ! fromIntegral (i .&. 0x3FFFFFFF))
         _          -> pprPanic "getSymtabName:unknown name tag" (ppr i)
 
 data BinSymbolTable = BinSymbolTable {
@@ -426,7 +420,6 @@ data BinDictionary = BinDictionary {
 -- All the binary instances
 
 -- BasicTypes
-{-! for IPName derive: Binary !-}
 {-! for Fixity derive: Binary !-}
 {-! for FixityDirection derive: Binary !-}
 {-! for Boxity derive: Binary !-}
@@ -618,16 +611,6 @@ instance Binary AvailInfo where
                       ac <- get bh
                       return (AvailTC ab ac)
 
-    
--- where should this be located?
-instance Binary ClockTime where
-    put_ bh (TOD x y) = put_ bh x >> put_ bh y
-    
-    get bh = do
-        x <- get bh
-        y <- get bh
-        return $ TOD x y
-
 instance Binary Usage where
     put_ bh usg@UsagePackageModule{} = do 
         putByte bh 0
@@ -835,11 +818,6 @@ instance Binary Fixity where
           ab <- get bh
           return (Fixity aa ab)
 
-instance (Binary name) => Binary (IPName name) where
-    put_ bh (IPName aa) = put_ bh aa
-    get bh = do aa <- get bh
-                return (IPName aa)
-
 -------------------------------------------------------------------------
 --              Types from: Demand
 -------------------------------------------------------------------------
@@ -1009,33 +987,14 @@ instance Binary IfaceType where
             putByte bh 3
             put_ bh ag
             put_ bh ah
-    
-        -- Simple compression for common cases of TyConApp
-    put_ bh (IfaceTyConApp IfaceIntTc  [])   = putByte bh 6
-    put_ bh (IfaceTyConApp IfaceCharTc [])   = putByte bh 7
-    put_ bh (IfaceTyConApp IfaceBoolTc [])   = putByte bh 8
-    put_ bh (IfaceTyConApp IfaceListTc [ty]) = do { putByte bh 9; put_ bh ty }
-        -- Unit tuple and pairs
-    put_ bh (IfaceTyConApp (IfaceTupTc BoxedTuple 0) []) = putByte bh 10
-    put_ bh (IfaceTyConApp (IfaceTupTc BoxedTuple 2) [t1,t2])
-      = do { putByte bh 11; put_ bh t1; put_ bh t2 }
-        -- Kind cases
-    put_ bh (IfaceTyConApp IfaceLiftedTypeKindTc [])   = putByte bh 12
-    put_ bh (IfaceTyConApp IfaceOpenTypeKindTc [])     = putByte bh 13
-    put_ bh (IfaceTyConApp IfaceUnliftedTypeKindTc []) = putByte bh 14
-    put_ bh (IfaceTyConApp IfaceUbxTupleKindTc [])     = putByte bh 15
-    put_ bh (IfaceTyConApp IfaceArgTypeKindTc [])      = putByte bh 16
-    put_ bh (IfaceTyConApp IfaceConstraintKindTc [])   = putByte bh 17
-    put_ bh (IfaceTyConApp IfaceSuperKindTc [])        = putByte bh 18
-
     put_ bh (IfaceCoConApp cc tys)
-      = do { putByte bh 19; put_ bh cc; put_ bh tys }
-
-        -- Generic cases
-    put_ bh (IfaceTyConApp (IfaceTc tc) tys)
-      = do { putByte bh 20; put_ bh tc; put_ bh tys }
+      = do { putByte bh 4; put_ bh cc; put_ bh tys }
     put_ bh (IfaceTyConApp tc tys)
-      = do { putByte bh 21; put_ bh tc; put_ bh tys }
+      = do { putByte bh 5; put_ bh tc; put_ bh tys }
+
+    put_ bh (IfaceLitTy n)
+      = do { putByte bh 30; put_ bh n }
+
 
     get bh = do
             h <- getByte bh
@@ -1051,70 +1010,32 @@ instance Binary IfaceType where
               3 -> do ag <- get bh
                       ah <- get bh
                       return (IfaceFunTy ag ah)
-              
-                -- Now the special cases for TyConApp
-              6 -> return (IfaceTyConApp IfaceIntTc [])
-              7 -> return (IfaceTyConApp IfaceCharTc [])
-              8 -> return (IfaceTyConApp IfaceBoolTc [])
-              9 -> do { ty <- get bh; return (IfaceTyConApp IfaceListTc [ty]) }
-              10 -> return (IfaceTyConApp (IfaceTupTc BoxedTuple 0) [])
-              11 -> do { t1 <- get bh; t2 <- get bh
-                       ; return (IfaceTyConApp (IfaceTupTc BoxedTuple 2) [t1,t2]) }
-              12 -> return (IfaceTyConApp IfaceLiftedTypeKindTc [])
-              13 -> return (IfaceTyConApp IfaceOpenTypeKindTc [])
-              14 -> return (IfaceTyConApp IfaceUnliftedTypeKindTc [])
-              15 -> return (IfaceTyConApp IfaceUbxTupleKindTc [])
-              16 -> return (IfaceTyConApp IfaceArgTypeKindTc [])
-              17 -> return (IfaceTyConApp IfaceConstraintKindTc [])
-              18 -> return (IfaceTyConApp IfaceSuperKindTc [])
+              4 -> do { cc <- get bh; tys <- get bh
+                      ; return (IfaceCoConApp cc tys) }
+              5 -> do { tc <- get bh; tys <- get bh
+                      ; return (IfaceTyConApp tc tys) }
 
-              19 -> do { cc <- get bh; tys <- get bh
-                        ; return (IfaceCoConApp cc tys) }
-
-              20 -> do { tc <- get bh; tys <- get bh
-                       ; return (IfaceTyConApp (IfaceTc tc) tys) }
-              21 -> do { tc <- get bh; tys <- get bh
-                        ; return (IfaceTyConApp tc tys) }
+              30 -> do n <- get bh
+                       return (IfaceLitTy n)
 
               _  -> panic ("get IfaceType " ++ show h)
 
-instance Binary IfaceTyCon where
-        -- Int,Char,Bool can't show up here because they can't not be saturated
-   put_ bh IfaceIntTc         = putByte bh 1
-   put_ bh IfaceBoolTc        = putByte bh 2
-   put_ bh IfaceCharTc        = putByte bh 3
-   put_ bh IfaceListTc        = putByte bh 4
-   put_ bh IfacePArrTc        = putByte bh 5
-   put_ bh IfaceLiftedTypeKindTc   = putByte bh 6
-   put_ bh IfaceOpenTypeKindTc     = putByte bh 7
-   put_ bh IfaceUnliftedTypeKindTc = putByte bh 8
-   put_ bh IfaceUbxTupleKindTc     = putByte bh 9
-   put_ bh IfaceArgTypeKindTc      = putByte bh 10
-   put_ bh IfaceConstraintKindTc   = putByte bh 11
-   put_ bh IfaceSuperKindTc        = putByte bh 12
-   put_ bh (IfaceTupTc bx ar)  = do { putByte bh 13; put_ bh bx; put_ bh ar }
-   put_ bh (IfaceTc ext)       = do { putByte bh 14; put_ bh ext }
-   put_ bh (IfaceIPTc n)       = do { putByte bh 15; put_ bh n }
+instance Binary IfaceTyLit where
+  put_ bh (IfaceNumTyLit n)  = putByte bh 1 >> put_ bh n
+  put_ bh (IfaceStrTyLit n)  = putByte bh 2 >> put_ bh n
 
-   get bh = do
-        h <- getByte bh
-        case h of
-          1 -> return IfaceIntTc
-          2 -> return IfaceBoolTc
-          3 -> return IfaceCharTc
-          4 -> return IfaceListTc
-          5 -> return IfacePArrTc
-          6 -> return IfaceLiftedTypeKindTc 
-          7 -> return IfaceOpenTypeKindTc 
-          8 -> return IfaceUnliftedTypeKindTc
-          9 -> return IfaceUbxTupleKindTc
-          10 -> return IfaceArgTypeKindTc
-          11 -> return IfaceConstraintKindTc
-          12 -> return IfaceSuperKindTc
-          13 -> do { bx <- get bh; ar <- get bh; return (IfaceTupTc bx ar) }
-          14 -> do { ext <- get bh; return (IfaceTc ext) }
-          15 -> do { n <- get bh; return (IfaceIPTc n) }
-          _  -> panic ("get IfaceTyCon " ++ show h)
+  get bh =
+    do tag <- getByte bh
+       case tag of
+         1 -> do { n <- get bh
+                 ; return (IfaceNumTyLit n) }
+         2 -> do { n <- get bh
+                 ; return (IfaceStrTyLit n) }
+         _ -> panic ("get IfaceTyLit " ++ show tag)
+
+instance Binary IfaceTyCon where
+   put_ bh (IfaceTc ext) = put_ bh ext
+   get bh = liftM IfaceTc (get bh)
 
 instance Binary IfaceCoCon where
    put_ bh (IfaceCoAx n)       = do { putByte bh 0; put_ bh n }
@@ -1124,8 +1045,7 @@ instance Binary IfaceCoCon where
    put_ bh IfaceTransCo        = putByte bh 4
    put_ bh IfaceInstCo         = putByte bh 5
    put_ bh (IfaceNthCo d)      = do { putByte bh 6; put_ bh d }
-   put_ bh (IfaceIPCoAx ip)    = do { putByte bh 7; put_ bh ip }
-  
+
    get bh = do
         h <- getByte bh
         case h of
@@ -1136,7 +1056,6 @@ instance Binary IfaceCoCon where
           4 -> return IfaceTransCo
           5 -> return IfaceInstCo
           6 -> do { d <- get bh; return (IfaceNthCo d) }
-          7 -> do { ip <- get bh; return (IfaceIPCoAx ip) }
           _ -> panic ("get IfaceCoCon " ++ show h)
 
 -------------------------------------------------------------------------
@@ -1192,6 +1111,10 @@ instance Binary IfaceExpr where
         putByte bh 12
         put_ bh ie
         put_ bh ico
+    put_ bh (IfaceECase a b) = do
+        putByte bh 13
+        put_ bh a
+        put_ bh b
     get bh = do
         h <- getByte bh
         case h of
@@ -1230,6 +1153,9 @@ instance Binary IfaceExpr where
             12 -> do ie <- get bh
                      ico <- get bh
                      return (IfaceCast ie ico)
+            13 -> do a <- get bh
+                     b <- get bh
+                     return (IfaceECase a b)
             _ -> panic ("get IfaceExpr " ++ show h)
 
 instance Binary IfaceConAlt where
@@ -1255,13 +1181,21 @@ instance Binary IfaceBinding where
 instance Binary IfaceIdDetails where
     put_ bh IfVanillaId      = putByte bh 0
     put_ bh (IfRecSelId a b) = putByte bh 1 >> put_ bh a >> put_ bh b
-    put_ bh IfDFunId         = putByte bh 2
+    put_ bh (IfDFunId n)     = do { putByte bh 2; put_ bh n }
     get bh = do
         h <- getByte bh
         case h of
             0 -> return IfVanillaId
             1 -> do { a <- get bh; b <- get bh; return (IfRecSelId a b) }
-            _ -> return IfDFunId
+            _ -> do { n <- get bh; return (IfDFunId n) }
+
+instance Binary (DFunArg IfaceExpr) where
+    put_ bh (DFunPolyArg  e) = putByte bh 0 >> put_ bh e
+    put_ bh (DFunLamArg i)   = putByte bh 1 >> put_ bh i
+    get bh = do { h <- getByte bh
+                ; case h of
+                    0 -> do { a <- get bh; return (DFunPolyArg a) }
+                    _ -> do { a <- get bh; return (DFunLamArg a) } }
 
 instance Binary IfaceIdInfo where
     put_ bh NoInfo      = putByte bh 0
@@ -1381,7 +1315,7 @@ instance Binary IfaceDecl where
     put_ _ (IfaceForeign _ _) = 
         error "Binary.put_(IfaceDecl): IfaceForeign"
 
-    put_ bh (IfaceData a1 a2 a3 a4 a5 a6 a7) = do
+    put_ bh (IfaceData a1 a2 a3 a4 a5 a6 a7 a8) = do
         putByte bh 2
         put_ bh (occNameFS a1)
         put_ bh a2
@@ -1390,14 +1324,14 @@ instance Binary IfaceDecl where
         put_ bh a5
         put_ bh a6
         put_ bh a7
+        put_ bh a8
 
-    put_ bh (IfaceSyn a1 a2 a3 a4 a5) = do
+    put_ bh (IfaceSyn a1 a2 a3 a4) = do
         putByte bh 3
         put_ bh (occNameFS a1)
         put_ bh a2
         put_ bh a3
         put_ bh a4
-        put_ bh a5
 
     put_ bh (IfaceClass a1 a2 a3 a4 a5 a6 a7) = do
         putByte bh 4
@@ -1408,6 +1342,13 @@ instance Binary IfaceDecl where
         put_ bh a5
         put_ bh a6
         put_ bh a7
+        
+    put_ bh (IfaceAxiom a1 a2 a3 a4) = do
+        putByte bh 5
+        put_ bh (occNameFS a1)
+        put_ bh a2
+        put_ bh a3
+        put_ bh a4
 
     get bh = do
         h <- getByte bh
@@ -1426,16 +1367,16 @@ instance Binary IfaceDecl where
                     a5 <- get bh
                     a6 <- get bh
                     a7 <- get bh
+                    a8 <- get bh
                     occ <- return $! mkOccNameFS tcName a1
-                    return (IfaceData occ a2 a3 a4 a5 a6 a7)
+                    return (IfaceData occ a2 a3 a4 a5 a6 a7 a8)
             3 -> do a1 <- get bh
                     a2 <- get bh
                     a3 <- get bh
                     a4 <- get bh
-                    a5 <- get bh
                     occ <- return $! mkOccNameFS tcName a1
-                    return (IfaceSyn occ a2 a3 a4 a5)
-            _ -> do a1 <- get bh
+                    return (IfaceSyn occ a2 a3 a4)
+            4 -> do a1 <- get bh
                     a2 <- get bh
                     a3 <- get bh
                     a4 <- get bh
@@ -1444,9 +1385,15 @@ instance Binary IfaceDecl where
                     a7 <- get bh
                     occ <- return $! mkOccNameFS clsName a2
                     return (IfaceClass a1 occ a3 a4 a5 a6 a7)
+            _ -> do a1 <- get bh
+                    a2 <- get bh
+                    a3 <- get bh
+                    a4 <- get bh
+                    occ <- return $! mkOccNameFS tcName a1
+                    return (IfaceAxiom occ a2 a3 a4)
 
-instance Binary IfaceInst where
-    put_ bh (IfaceInst cls tys dfun flag orph) = do
+instance Binary IfaceClsInst where
+    put_ bh (IfaceClsInst cls tys dfun flag orph) = do
         put_ bh cls
         put_ bh tys
         put_ bh dfun
@@ -1458,18 +1405,20 @@ instance Binary IfaceInst where
         dfun <- get bh
         flag <- get bh
         orph <- get bh
-        return (IfaceInst cls tys dfun flag orph)
+        return (IfaceClsInst cls tys dfun flag orph)
 
 instance Binary IfaceFamInst where
-    put_ bh (IfaceFamInst fam tys tycon) = do
+    put_ bh (IfaceFamInst fam tys name orph) = do
         put_ bh fam
         put_ bh tys
-        put_ bh tycon
+        put_ bh name
+        put_ bh orph
     get bh = do
-        fam   <- get bh
-        tys   <- get bh
-        tycon <- get bh
-        return (IfaceFamInst fam tys tycon)
+        fam      <- get bh
+        tys      <- get bh
+        name     <- get bh
+        orph     <- get bh
+        return (IfaceFamInst fam tys name orph)
 
 instance Binary OverlapFlag where
     put_ bh (NoOverlap  b) = putByte bh 0 >> put_ bh b
@@ -1486,14 +1435,14 @@ instance Binary OverlapFlag where
 
 instance Binary IfaceConDecls where
     put_ bh (IfAbstractTyCon d) = putByte bh 0 >> put_ bh d
-    put_ bh IfOpenDataTyCon     = putByte bh 1
+    put_ bh IfDataFamTyCon     = putByte bh 1
     put_ bh (IfDataTyCon cs)    = putByte bh 2 >> put_ bh cs
     put_ bh (IfNewTyCon c)      = putByte bh 3 >> put_ bh c
     get bh = do
         h <- getByte bh
         case h of
             0 -> get bh >>= (return . IfAbstractTyCon)
-            1 -> return IfOpenDataTyCon
+            1 -> return IfDataFamTyCon
             2 -> get bh >>= (return . IfDataTyCon)
             _ -> get bh >>= (return . IfNewTyCon)
 
