@@ -14,8 +14,6 @@
 -- for details
 
 module PprBase (
-	asmSDoc,
-	pprCLabel_asm,
 	castFloatToWord8Array,
 	castDoubleToWord8Array,
 	floatToBytes,
@@ -24,11 +22,6 @@ module PprBase (
 
 where
 
-import qualified Outputable
-import Platform
-import CLabel
-import Pretty
-
 -- castSTUArray has moved to Data.Array.Unsafe
 #if __GLASGOW_HASKELL__ >= 703
 import Data.Array.Unsafe( castSTUArray )
@@ -43,16 +36,6 @@ import Data.Word
 
 
 
-asmSDoc :: Outputable.SDoc -> Doc
-asmSDoc d 
-	= Outputable.withPprStyleDoc (Outputable.mkCodeStyle Outputable.AsmStyle) d
-
-
-pprCLabel_asm :: Platform -> CLabel -> Doc
-pprCLabel_asm platform l
-    = asmSDoc (pprCLabel platform l)
-
-
 -- -----------------------------------------------------------------------------
 -- Converting floating-point literals to integrals for printing
 
