@@ -22,17 +22,15 @@ import Instruction
 import OldCmm
 
 import Outputable
-import Platform
 
 
 -- | Enforce intra-block invariants.
 --
-checkBlock :: Platform
-           -> CmmBasicBlock
+checkBlock :: CmmBasicBlock
            -> NatBasicBlock Instr
            -> NatBasicBlock Instr
 
-checkBlock platform cmm block@(BasicBlock _ instrs)
+checkBlock cmm block@(BasicBlock _ instrs)
 	| checkBlockInstrs instrs
 	= block
 	
@@ -40,9 +38,9 @@ checkBlock platform cmm block@(BasicBlock _ instrs)
 	= pprPanic 
 		("SPARC.CodeGen: bad block\n")
 		( vcat	[ text " -- cmm -----------------\n"
-			, pprPlatform platform cmm
+			, ppr cmm
 			, text " -- native code ---------\n"
-			, pprPlatform platform block ])
+			, ppr block ])
 
 
 checkBlockInstrs :: [Instr] -> Bool
