@@ -1,7 +1,3 @@
-{-# LANGUAGE CPP #-}
-#if sh_SAFE_DEFAULT
-{-# LANGUAGE Safe #-}
-#endif
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Monad.ST.Strict
@@ -20,9 +16,5 @@ module Control.Monad.ST.Strict (
         module Control.Monad.ST
   ) where
 
-#if sh_SAFE_DEFAULT
-import safe Control.Monad.ST
-#else
 import Control.Monad.ST
-#endif
 
