@@ -1,7 +1,5 @@
-{-# LANGUAGE SafeImports, CPP, NoImplicitPrelude #-}
-#if sh_SAFE_DEFAULT
-{-# LANGUAGE Trustworthy #-}
-#endif
+{-# LANGUAGE Unsafe #-}
+{-# LANGUAGE CPP, NoImplicitPrelude #-}
 {-# OPTIONS_HADDOCK hide #-}
 
 -----------------------------------------------------------------------------
@@ -21,16 +19,41 @@
 -----------------------------------------------------------------------------
 
 module Foreign.ForeignPtr ( 
-          module Foreign.ForeignPtr.Safe
-#if !sh_SAFE_DEFAULT
+        -- * Finalised data pointers
+          ForeignPtr
+        , FinalizerPtr
+#if defined(__HUGS__) || defined(__GLASGOW_HASKELL__)
+        , FinalizerEnvPtr
+#endif
+        -- ** Basic operations
+        , newForeignPtr
+        , newForeignPtr_
+        , addForeignPtrFinalizer
+#if defined(__HUGS__) || defined(__GLASGOW_HASKELL__)
+        , newForeignPtrEnv
+        , addForeignPtrFinalizerEnv
+#endif
+        , withForeignPtr
+
+#ifdef __GLASGOW_HASKELL__
+        , finalizeForeignPtr
+#endif
+
+        -- ** Low-level operations
+        , touchForeignPtr
+        , castForeignPtr
+
+        -- ** Allocating managed memory
+        , mallocForeignPtr
+        , mallocForeignPtrBytes
+        , mallocForeignPtrArray
+        , mallocForeignPtrArray0
         -- ** Unsafe low-level operations
         , unsafeForeignPtrToPtr
-#endif
     ) where
 
-import safe Foreign.ForeignPtr.Safe
+import Foreign.ForeignPtr.Safe
 
-#if !sh_SAFE_DEFAULT
 import Foreign.Ptr ( Ptr )
 import qualified Foreign.ForeignPtr.Unsafe as U
 
@@ -38,5 +61,4 @@ import qualified Foreign.ForeignPtr.Unsafe as U
 {-# INLINE unsafeForeignPtrToPtr #-}
 unsafeForeignPtrToPtr :: ForeignPtr a -> Ptr a
 unsafeForeignPtrToPtr = U.unsafeForeignPtrToPtr
-#endif
 
