@@ -122,7 +122,11 @@ ioManager = unsafePerformIO $ do
 ensureIOManagerIsRunning :: IO ()
 ensureIOManagerIsRunning
   | not threaded = return ()
-  | otherwise = modifyMVar_ ioManager $ \old -> do
+  | otherwise = do
+      startIOManagerThread
+
+startIOManagerThread :: IO ()
+startIOManagerThread = modifyMVar_ ioManager $ \old -> do
   let create = do
         !mgr <- new
         writeIORef eventManager $ Just mgr
