@@ -1,3 +1,4 @@
+{-# LANGUAGE Unsafe #-}
 {-# LANGUAGE CPP, NoImplicitPrelude #-}
 
 -----------------------------------------------------------------------------
@@ -19,10 +20,15 @@ module System.IO.Unsafe (
    unsafePerformIO,     -- :: IO a -> a
    unsafeDupablePerformIO, -- :: IO a -> a
    unsafeInterleaveIO,  -- :: IO a -> IO a
+   unsafeFixIO,
   ) where
 
 #ifdef __GLASGOW_HASKELL__
-import GHC.IO      (unsafePerformIO, unsafeInterleaveIO, unsafeDupablePerformIO)
+import GHC.Base
+import GHC.IO
+import GHC.IORef
+import GHC.Exception
+import Control.Exception
 #endif
 
 #ifdef __HUGS__
@@ -35,3 +41,21 @@ import NHC.Internal (unsafePerformIO, unsafeInterleaveIO)
 unsafeDupablePerformIO = unsafePerformIO
 #endif
 
+-- | A slightly faster version of `System.IO.fixIO` that may not be
+-- safe to use with multiple threads.  The unsafety arises when used
+-- like this:
+--
+-- >  unsafeFixIO $ \r ->
+-- >     forkIO (print r)
+-- >     return (...)
+--
+-- In this case, the child thread will receive a @NonTermination@
+-- exception instead of waiting for the value of @r@ to be computed.
+--
+unsafeFixIO :: (a -> IO a) -> IO a
+unsafeFixIO k = do
+  ref <- newIORef (throw NonTermination)
+  ans <- unsafeDupableInterleaveIO (readIORef ref)
+  result <- k ans
+  writeIORef ref result
+  return result
