@@ -94,15 +94,10 @@ mkWeakPtr key finalizer = mkWeak key key finalizer
   when the key becomes unreachable).
 
   Note: adding a finalizer to a 'Foreign.ForeignPtr.ForeignPtr' using
-  'addFinalizer' won't work as well as using the specialised version
-  'Foreign.ForeignPtr.addForeignPtrFinalizer' because the latter
-  version adds the finalizer to the primitive 'ForeignPtr#' object
-  inside, whereas the generic 'addFinalizer' will add the finalizer to
-  the box.  Optimisations tend to remove the box, which may cause the
-  finalizer to run earlier than you intended.  The same motivation
-  justifies the existence of
-  'Control.Concurrent.MVar.addMVarFinalizer' and
-  'Data.IORef.mkWeakIORef' (the non-uniformity is accidental).
+  'addFinalizer' won't work; use the specialised version
+  'Foreign.ForeignPtr.addForeignPtrFinalizer' instead.  For discussion
+  see the 'Weak' type.
+.
 -}
 addFinalizer :: key -> IO () -> IO ()
 addFinalizer key finalizer = do
@@ -139,8 +134,9 @@ The behaviour is simply this:
 This behaviour depends on what it means for a key to be reachable.
 Informally, something is reachable if it can be reached by following
 ordinary pointers from the root set, but not following weak pointers.
-We define reachability more precisely as follows A heap object is
-reachable if:
+We define reachability more precisely as follows.
+
+A heap object is /reachable/ if:
 
  * It is a member of the /root set/.
 
@@ -149,6 +145,6 @@ reachable if:
 
  * It is a weak pointer object whose key is reachable.
 
- * It is the value or finalizer of an object whose key is reachable.
+ * It is the value or finalizer of a weak pointer object whose key is reachable.
 -}
 
