@@ -8,9 +8,6 @@ import qualified Data.Sequence as S
 import qualified Data.Foldable
 import System.Random
 
-instance NFData a => NFData (S.Seq a) where
-    rnf = Data.Foldable.foldr seq ()
-
 main = do
     let s10 = S.fromList [1..10] :: S.Seq Int
         s100 = S.fromList [1..100] :: S.Seq Int
