@@ -1,9 +1,10 @@
+{-# LANGUAGE Trustworthy #-}
 
-module GHC.Debug (debugLn, debugErrLn) where
+module GHC.Debug ( debugLn, debugErrLn ) where
 
 import GHC.Prim
 import GHC.Types
-import GHC.Unit ()
+import GHC.Tuple ()
 
 debugLn :: [Char] -> IO ()
 debugLn xs = IO (\s0 ->
