@@ -1,3 +1,7 @@
+#if __GLASGOW_HASKELL__ >= 701
+{-# LANGUAGE Safe #-}
+#endif
+
 module System.Environment (
       getArgs,       -- :: IO [String]
       getProgName,   -- :: IO String
