@@ -4,6 +4,7 @@
 
 module Compiler.Hoopl
   ( module Compiler.Hoopl.Graph
+  , module Compiler.Hoopl.Block
   , module Compiler.Hoopl.MkGraph
   , module Compiler.Hoopl.XUtil
   , module Compiler.Hoopl.Collections
@@ -14,7 +15,6 @@ module Compiler.Hoopl
   , module Compiler.Hoopl.Combinators
   , module Compiler.Hoopl.Fuel
   , module Compiler.Hoopl.Unique
-  , module Compiler.Hoopl.Util
   , module Compiler.Hoopl.Debug
   , module Compiler.Hoopl.Show
   )
@@ -27,15 +27,11 @@ import Compiler.Hoopl.Dataflow hiding ( wrapFR, wrapFR2, wrapBR, wrapBR2
                                       )
 import Compiler.Hoopl.Debug
 import Compiler.Hoopl.Fuel hiding (withFuel, getFuel, setFuel, runWithFuel)
-import Compiler.Hoopl.Graph hiding 
-   ( Body
-   , BCat, BHead, BTail, BClosed -- OK to expose BFirst, BMiddle, BLast
-   )
-import Compiler.Hoopl.Graph (Body)
+import Compiler.Hoopl.Block
+import Compiler.Hoopl.Graph hiding (splice, gSplice)
 import Compiler.Hoopl.Label hiding (uniqueToLbl, lblToUnique)
 import Compiler.Hoopl.MkGraph
 import Compiler.Hoopl.Pointed
 import Compiler.Hoopl.Show
-import Compiler.Hoopl.Util
 import Compiler.Hoopl.Unique hiding (uniqueToInt)
 import Compiler.Hoopl.XUtil
