@@ -1,4 +1,8 @@
 {-# LANGUAGE GADTs #-}
+#if __GLASGOW_HASKELL__ >= 701
+{-# LANGUAGE Safe #-}
+#endif
+
 module Haddock
 where
 
