@@ -9,6 +9,7 @@ module Compiler.Hoopl.Show
 where
 
 import Compiler.Hoopl.Collections
+import Compiler.Hoopl.Block
 import Compiler.Hoopl.Graph
 import Compiler.Hoopl.Label
 
@@ -28,13 +29,14 @@ showGraph node = g
             open b g_entry ++ body g_blocks ++ open b g_exit
         body blocks = concatMap b (mapElems blocks)
         b :: forall e x . Block n e x -> String
-        b (BFirst  n)     = node n
-        b (BMiddle n)     = node n
-        b (BLast   n)     = node n ++ "\n"
+        b (BlockCO l b1)   = node l ++ "\n" ++ b b1
+        b (BlockCC l b1 n) = node l ++ "\n" ++ b b1 ++ node n ++ "\n"
+        b (BlockOC   b1 n) =           b b1 ++ node n ++ "\n"
+        b (BNil)          = ""
+        b (BMiddle n)     = node n ++ "\n"
         b (BCat b1 b2)    = b b1   ++ b b2
-        b (BHead b1 n)    = b b1   ++ node n ++ "\n"
-        b (BTail n b1)    = node n ++ b b1
-        b (BClosed b1 b2) = b b1   ++ b b2
+        b (BSnoc b1 n)    = b b1   ++ node n ++ "\n"
+        b (BCons n b1)    = node n ++ "\n" ++ b b1
 
 open :: (a -> String) -> MaybeO z a -> String
 open _ NothingO  = ""
