@@ -1,6 +1,7 @@
 module Main (main) where
 
 import Test
+import System.IO
 
 -- Hardcoding test locations for now
 tests = map (\t -> "tests" ++ "/" ++ t)
@@ -8,5 +9,7 @@ tests = map (\t -> "tests" ++ "/" ++ t)
              ["if-test", "if-test2", "if-test3", "if-test4"])
 
 main :: IO ()
-main = do mapM (\x -> putStrLn ("Test:" ++ x) >> parseTest x >> optTest x) tests
+main = do hSetBuffering stdout NoBuffering
+          hSetBuffering stderr NoBuffering
+          mapM (\x -> putStrLn ("Test:" ++ x) >> parseTest x >> optTest x) tests
           return ()
