@@ -35,8 +35,9 @@ runInteractiveProcess (char *const args[],
 		       int *pfdStdInput, int *pfdStdOutput, int *pfdStdError,
                        int set_inthandler, long inthandler, 
                        int set_quithandler, long quithandler,
-                       int close_fds)
+                       int flags)
 {
+    int close_fds = ((flags & RUN_PROCESS_IN_CLOSE_FDS) != 0);
     int pid;
     int fdStdInput[2], fdStdOutput[2], fdStdError[2];
     int r;
@@ -103,6 +104,10 @@ runInteractiveProcess (char *const args[],
         // WARNING!  we are now in the child of vfork(), so any memory
         // we modify below will also be seen in the parent process.
 
+        if ((flags & RUN_PROCESS_IN_NEW_GROUP) != 0) {
+            setpgid(0, 0);
+        }
+
         unblockUserSignals();
 
 	if (workingDirectory) {
@@ -191,6 +196,9 @@ runInteractiveProcess (char *const args[],
     _exit(127);
     
     default:
+	if ((flags & RUN_PROCESS_IN_NEW_GROUP) != 0) {
+            setpgid(pid, pid);
+	}
 	if (fdStdIn  == -1) {
             close(fdStdInput[0]);
             fcntl(fdStdInput[1], F_SETFD, FD_CLOEXEC);
@@ -261,12 +269,9 @@ int waitForProcess (ProcHandle handle, int *pret)
 {
     int wstat;
     
-    while (waitpid(handle, &wstat, 0) < 0)
+    if (waitpid(handle, &wstat, 0) < 0)
     {
-	if (errno != EINTR)
-	{
-	    return -1;
-	}
+        return -1;
     }
     
     if (WIFEXITED(wstat)) {
@@ -352,10 +357,10 @@ mkAnonPipe (HANDLE* pHandleIn, BOOL isInheritableIn,
 
 ProcHandle
 runInteractiveProcess (wchar_t *cmd, wchar_t *workingDirectory, 
-                       void *environment,
+                       wchar_t *environment,
                        int fdStdIn, int fdStdOut, int fdStdErr,
 		       int *pfdStdInput, int *pfdStdOutput, int *pfdStdError,
-                       int close_fds)
+                       int flags)
 {
 	STARTUPINFO sInfo;
 	PROCESS_INFORMATION pInfo;
@@ -365,7 +370,9 @@ runInteractiveProcess (wchar_t *cmd, wchar_t *workingDirectory,
         HANDLE hStdOutputWrite = INVALID_HANDLE_VALUE;
 	HANDLE hStdErrorRead   = INVALID_HANDLE_VALUE;
         HANDLE hStdErrorWrite  = INVALID_HANDLE_VALUE;
-	DWORD flags;
+    BOOL close_fds = ((flags & RUN_PROCESS_IN_CLOSE_FDS) != 0);
+	// We always pass a wide environment block, so we MUST set this flag 
+        DWORD dwFlags = CREATE_UNICODE_ENVIRONMENT;
 	BOOL status;
         BOOL inherit;
 
@@ -433,12 +440,11 @@ runInteractiveProcess (wchar_t *cmd, wchar_t *workingDirectory,
             sInfo.hStdError = hStdErrorWrite;
         }
 
-	if (sInfo.hStdInput  != GetStdHandle(STD_INPUT_HANDLE)  &&
+        if (sInfo.hStdInput  != GetStdHandle(STD_INPUT_HANDLE)  &&
 	    sInfo.hStdOutput != GetStdHandle(STD_OUTPUT_HANDLE) &&
-	    sInfo.hStdError  != GetStdHandle(STD_ERROR_HANDLE))
-		flags = CREATE_NO_WINDOW;   // Run without console window only when both output and error are redirected
-	else
-		flags = 0;
+	    sInfo.hStdError  != GetStdHandle(STD_ERROR_HANDLE)  &&
+	    (flags & RUN_PROCESS_IN_NEW_GROUP) == 0)
+		dwFlags |= CREATE_NO_WINDOW;   // Run without console window only when both output and error are redirected
 
         // See #3231
         if (close_fds && fdStdIn == 0 && fdStdOut == 1 && fdStdErr == 2) {
@@ -446,8 +452,12 @@ runInteractiveProcess (wchar_t *cmd, wchar_t *workingDirectory,
         } else {
             inherit = TRUE;
         }
+ 
+        if ((flags & RUN_PROCESS_IN_NEW_GROUP) != 0) {
+            dwFlags |= CREATE_NEW_PROCESS_GROUP;
+        }
 
-	if (!CreateProcess(NULL, cmd, NULL, NULL, inherit, flags, environment, workingDirectory, &sInfo, &pInfo))
+	if (!CreateProcess(NULL, cmd, NULL, NULL, inherit, dwFlags, environment, workingDirectory, &sInfo, &pInfo))
 	{
                 goto cleanup_err;
 	}
@@ -464,7 +474,7 @@ runInteractiveProcess (wchar_t *cmd, wchar_t *workingDirectory,
 	*pfdStdOutput = _open_osfhandle((intptr_t) hStdOutputRead, _O_RDONLY);
   	*pfdStdError  = _open_osfhandle((intptr_t) hStdErrorRead,  _O_RDONLY);
 
-  	return (int) pInfo.hProcess;
+  	return pInfo.hProcess;
 
 cleanup_err:
         if (hStdInputRead   != INVALID_HANDLE_VALUE) CloseHandle(hStdInputRead);
@@ -474,7 +484,7 @@ cleanup_err:
         if (hStdErrorRead   != INVALID_HANDLE_VALUE) CloseHandle(hStdErrorRead);
         if (hStdErrorWrite  != INVALID_HANDLE_VALUE) CloseHandle(hStdErrorWrite);
         maperrno();
-        return -1;
+        return NULL;
 }
 
 int
