@@ -134,6 +134,19 @@ int __hscore_mkstemp(char *filetemplate) {
 }
 #endif
 
+#if HAVE_MKSTEMPS
+int __hscore_mkstemps(char *filetemplate, int suffixlen) {
+    return (mkstemps(filetemplate, suffixlen));
+}
+#endif
+
+#if HAVE_MKDTEMP
+char *__hscore_mkdtemp(char *filetemplate) {
+    return (mkdtemp(filetemplate));
+}
+#endif
+
+
 #if !defined(__MINGW32__) && !defined(irix_HOST_OS)
 int __hscore_getrlimit(int resource, struct rlimit *rlim) {
     return (getrlimit(resource, rlim));
