@@ -11,6 +11,8 @@
 
 #include "BeginPrivate.h"
 
+void initializeTimer       (void);
+
 Time getProcessCPUTime     (void);
 Time getThreadCPUTime      (void);
 Time getProcessElapsedTime (void);
@@ -22,7 +24,7 @@ void getProcessTimes       (Time *user, Time *elapsed);
 void  getUnixEpochTime      (StgWord64 *sec, StgWord32 *nsec);
 
 // Not strictly timing, but related
-nat   getPageFaults         (void);
+W_    getPageFaults         (void);
 
 #include "EndPrivate.h"
 
