@@ -40,6 +40,9 @@ void postSchedEvent(Capability *cap, EventTypeNum tag,
  */
 void postEvent(Capability *cap, EventTypeNum tag);
 
+void postEventAtTimestamp (Capability *cap, EventTimestamp ts, 
+                           EventTypeNum tag);
+
 void postMsg(char *msg, va_list ap);
 
 void postUserMsg(Capability *cap, char *msg, va_list ap);
@@ -48,6 +51,12 @@ void postCapMsg(Capability *cap, char *msg, va_list ap);
 
 void postEventStartup(EventCapNo n_caps);
 
+/*
+ * Post an event relating to a capability itself (create/delete/etc)
+ */
+void postCapEvent (EventTypeNum  tag,
+                   EventCapNo    capno);
+
 /*
  * Post an event that is associated with a capability set
  */
@@ -91,6 +100,31 @@ void postThreadLabel(Capability    *cap,
                      EventThreadID  id,
                      char          *label);
 
+/*
+ * Various GC and heap events
+ */
+void postHeapEvent (Capability    *cap,
+                    EventTypeNum   tag,
+                    EventCapsetID  heap_capset,
+                    W_           info1);
+
+void postEventHeapInfo (EventCapsetID heap_capset,
+                        nat           gens,
+                        W_          maxHeapSize,
+                        W_          allocAreaSize,
+                        W_          mblockSize,
+                        W_          blockSize);
+
+void postEventGcStats  (Capability    *cap,
+                        EventCapsetID  heap_capset,
+                        nat            gen,
+                        W_           copied,
+                        W_           slop,
+                        W_           fragmentation,
+                        nat            par_n_threads,
+                        W_           par_max_copied,
+                        W_           par_tot_copied);
+
 #else /* !TRACING */
 
 INLINE_HEADER void postSchedEvent (Capability *cap  STG_UNUSED,
