@@ -12,6 +12,6 @@
 void
 StackOverflowHook (lnat stack_size)    /* in bytes */
 {
-    fprintf(stderr, "Stack space overflow: current size %ld bytes.\nUse `+RTS -Ksize -RTS' to increase it.\n", stack_size);
+    fprintf(stderr, "Stack space overflow: current size %" FMT_Word " bytes.\nUse `+RTS -Ksize -RTS' to increase it.\n", stack_size);
 }
 
