@@ -24,7 +24,7 @@
 #include "Itimer.h"
 #include "Proftimer.h"
 #include "Schedule.h"
-#include "Select.h"
+#include "Clock.h"
 
 /* As recommended in the autoconf manual */
 # ifdef TIME_WITH_SYS_TIME
@@ -123,21 +123,14 @@ initTicker (Time interval, TickProc handle_tick)
 #if defined(USE_TIMER_CREATE)
     {
         struct sigevent ev;
-        clockid_t clock;
 
         // Keep programs like valgrind happy
-	memset(&ev, 0, sizeof(ev));
+        memset(&ev, 0, sizeof(ev));
 
         ev.sigev_notify = SIGEV_SIGNAL;
         ev.sigev_signo  = ITIMER_SIGNAL;
 
-#if defined(CLOCK_MONOTONIC)
-        clock = CLOCK_MONOTONIC;
-#else
-        clock = CLOCK_REALTIME;
-#endif
-
-        if (timer_create(clock, &ev, &timer) != 0) {
+        if (timer_create(CLOCK_ID, &ev, &timer) != 0) {
             sysErrorBelch("timer_create");
             stg_exit(EXIT_FAILURE);
         }
@@ -155,7 +148,7 @@ startTicker(void)
         struct itimerspec it;
         
         it.it_value.tv_sec  = TimeToSeconds(itimer_interval);
-        it.it_value.tv_nsec = TimeToNS(itimer_interval);
+        it.it_value.tv_nsec = TimeToNS(itimer_interval) % 1000000000;
         it.it_interval = it.it_value;
         
         if (timer_settime(timer, 0, &it, NULL) != 0) {
@@ -168,7 +161,7 @@ startTicker(void)
         struct itimerval it;
 
         it.it_value.tv_sec = TimeToSeconds(itimer_interval);
-        it.it_value.tv_usec = TimeToUS(itimer_interval);
+        it.it_value.tv_usec = TimeToUS(itimer_interval) % 1000000;
         it.it_interval = it.it_value;
         
         if (setitimer(ITIMER_REAL, &it, NULL) != 0) {
@@ -211,6 +204,9 @@ void
 exitTicker (rtsBool wait STG_UNUSED)
 {
 #if defined(USE_TIMER_CREATE)
+    // Before deleting the timer set the signal to ignore to avoid the
+    // possibility of the signal being delivered after the timer is deleted.
+    signal(ITIMER_SIGNAL, SIG_IGN);
     timer_delete(timer);
     // ignore errors - we don't really care if it fails.
 #endif
