@@ -20,6 +20,7 @@
 #include "Select.h"
 #include "AwaitEvent.h"
 #include "Stats.h"
+#include "GetTime.h"
 
 # ifdef HAVE_SYS_SELECT_H
 #  include <sys/select.h>
@@ -29,16 +30,10 @@
 #  include <sys/types.h>
 # endif
 
-# ifdef HAVE_SYS_TIME_H
-#  include <sys/time.h>
-# endif
-
 #include <errno.h>
 #include <string.h>
 
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
+#include "Clock.h"
 
 #if !defined(THREADED_RTS)
 
@@ -50,7 +45,7 @@
 
 #if SIZEOF_VOID_P == 4
 #define LowResTimeToTime(t)          (USToTime((t) * 1000))
-#define TimeToLowResTimeRoundDown(t) (TimeToUS(t) / 1000)
+#define TimeToLowResTimeRoundDown(t) ((LowResTime)(TimeToUS(t) / 1000))
 #define TimeToLowResTimeRoundUp(t)   ((TimeToUS(t) + 1000-1) / 1000)
 #else
 #define LowResTimeToTime(t) (t)
@@ -64,7 +59,7 @@
  */
 static LowResTime getLowResTimeOfDay(void)
 {
-    return TimeToLowResTimeRoundDown(stat_getElapsedTime());
+    return TimeToLowResTimeRoundDown(getProcessElapsedTime());
 }
 
 /*
@@ -72,9 +67,18 @@ static LowResTime getLowResTimeOfDay(void)
  */
 LowResTime getDelayTarget (HsInt us)
 {
-    // round up the target time, because we never want to sleep *less*
-    // than the desired amount.
-    return TimeToLowResTimeRoundUp(stat_getElapsedTime() + USToTime(us));
+    Time elapsed;
+    elapsed = getProcessElapsedTime();
+
+    // If the desired target would be larger than the maximum Time,
+    // default to the maximum Time. (#7087)
+    if (us > TimeToUS(TIME_MAX - elapsed)) {
+        return TimeToLowResTimeRoundDown(TIME_MAX);
+    } else {
+        // round up the target time, because we never want to sleep *less*
+        // than the desired amount.
+        return TimeToLowResTimeRoundUp(elapsed + USToTime(us));
+    }
 }
 
 /* There's a clever trick here to avoid problems when the time wraps
@@ -217,19 +221,10 @@ awaitEvent(rtsBool wait)
           ptv = NULL;
       }
 
-      while (1) { // repeat the select on EINTR
-
-          // Disable the timer signal while blocked in
-          // select(), to conserve power. (#1623, #5991)
-          if (wait) stopTimer();
-
-          numFound = select(maxfd+1, &rfd, &wfd, NULL, ptv);
-
-          if (wait) startTimer();
-
-          if (numFound >= 0) break;
-
-          if (errno != EINTR) {
+      /* Check for any interesting events */
+      
+      while ((numFound = select(maxfd+1, &rfd, &wfd, NULL, ptv)) < 0) {
+	  if (errno != EINTR) {
 	    /* Handle bad file descriptors by unblocking all the
 	       waiting threads. Why? Because a thread might have been
 	       a bit naughty and closed a file descriptor while another
@@ -248,12 +243,12 @@ awaitEvent(rtsBool wait)
 	       the RTS won't loop.
 	    */
 	    if ( errno == EBADF ) {
-	      unblock_all = rtsTrue;
-	      break;
+                unblock_all = rtsTrue;
+                break;
 	    } else {
- 	      perror("select");
-	      barf("select failed");
-	    }
+                sysErrorBelch("select");
+                stg_exit(EXIT_FAILURE);
+            }
 	  }
 
 	  /* We got a signal; could be one of ours.  If so, we need
@@ -277,7 +272,7 @@ awaitEvent(rtsBool wait)
 	  /* check for threads that need waking up 
 	   */
           wakeUpSleepingThreads(getLowResTimeOfDay());
-	  
+
 	  /* If new runnable threads have arrived, stop waiting for
 	   * I/O and run them.
 	   */
