@@ -11,21 +11,23 @@
 
 #include "BeginPrivate.h"
 
+bdescr *allocLargeChunk (nat min, nat max);
+
 /* Debugging  -------------------------------------------------------------- */
 
-extern nat countBlocks       (bdescr *bd);
-extern nat countAllocdBlocks (bdescr *bd);
+extern W_ countBlocks       (bdescr *bd);
+extern W_ countAllocdBlocks (bdescr *bd);
 extern void returnMemoryToOS(nat n);
 
 #ifdef DEBUG
 void checkFreeListSanity(void);
-nat  countFreeList(void);
+W_   countFreeList(void);
 void markBlocks (bdescr *bd);
 void reportUnmarkedBlocks (void);
 #endif
 
-extern lnat n_alloc_blocks;   // currently allocated blocks
-extern lnat hw_alloc_blocks;  // high-water allocated blocks
+extern W_ n_alloc_blocks;   // currently allocated blocks
+extern W_ hw_alloc_blocks;  // high-water allocated blocks
 
 #include "EndPrivate.h"
 
