@@ -14,7 +14,7 @@
 #ifndef SM_MARKWEAK_H
 #define SM_MARKWEAK_H
 
-BEGIN_RTS_PRIVATE
+#include "BeginPrivate.h"
 
 extern StgWeak *old_weak_ptr_list;
 extern StgTSO *resurrected_threads;
@@ -23,8 +23,7 @@ extern StgTSO *exception_threads;
 void    initWeakForGC          ( void );
 rtsBool traverseWeakPtrList    ( void );
 void    markWeakPtrList        ( void );
-rtsBool traverseBlackholeQueue ( void );
 
-END_RTS_PRIVATE
+#include "EndPrivate.h"
 
 #endif /* SM_MARKWEAK_H */
