@@ -1,6 +1,6 @@
 /* -----------------------------------------------------------------------------
  *
- * (c) The GHC Team, 1998-2008
+ * (c) The GHC Team, 1998-2012
  *
  * Storage manager front end
  *
@@ -24,7 +24,7 @@
 #include "Arena.h"
 #include "Capability.h"
 #include "Schedule.h"
-#include "RetainerProfile.h"	// for counting memory blocks (memInventory)
+#include "RetainerProfile.h"        // for counting memory blocks (memInventory)
 #include "OSMem.h"
 #include "Trace.h"
 #include "GC.h"
@@ -41,13 +41,13 @@ StgClosure    *caf_list         = NULL;
 StgClosure    *revertible_caf_list = NULL;
 rtsBool       keepCAFs;
 
-nat large_alloc_lim;    /* GC if n_large_blocks in any nursery
-                         * reaches this. */
+W_ large_alloc_lim;    /* GC if n_large_blocks in any nursery
+                        * reaches this. */
 
 bdescr *exec_block;
 
-generation *generations = NULL;	/* all the generations */
-generation *g0		= NULL; /* generation 0, for convenience */
+generation *generations = NULL; /* all the generations */
+generation *g0          = NULL; /* generation 0, for convenience */
 generation *oldest_gen  = NULL; /* oldest generation, for convenience */
 
 nursery *nurseries = NULL;     /* array of nurseries, size == n_capabilities */
@@ -78,6 +78,7 @@ initGeneration (generation *gen, int g)
     gen->n_old_blocks = 0;
     gen->large_objects = NULL;
     gen->n_large_blocks = 0;
+    gen->n_large_words = 0;
     gen->n_new_large_words = 0;
     gen->scavenged_large_objects = NULL;
     gen->n_scavenged_large_blocks = 0;
@@ -92,7 +93,7 @@ initGeneration (generation *gen, int g)
 }
 
 void
-initStorage( void )
+initStorage (void)
 {
   nat g;
 
@@ -114,7 +115,7 @@ initStorage( void )
   if (RtsFlags.GcFlags.maxHeapSize != 0 &&
       RtsFlags.GcFlags.heapSizeSuggestion > 
       RtsFlags.GcFlags.maxHeapSize) {
-    RtsFlags.GcFlags.maxHeapSize = RtsFlags.GcFlags.heapSizeSuggestion;
+      RtsFlags.GcFlags.maxHeapSize = RtsFlags.GcFlags.heapSizeSuggestion;
   }
 
   if (RtsFlags.GcFlags.maxHeapSize != 0 &&
@@ -134,8 +135,8 @@ initStorage( void )
 
   /* allocate generation info array */
   generations = (generation *)stgMallocBytes(RtsFlags.GcFlags.generations 
-					     * sizeof(struct generation_),
-					     "initStorage: gens");
+                                             * sizeof(struct generation_),
+                                             "initStorage: gens");
 
   /* Initialise all generations */
   for(g = 0; g < RtsFlags.GcFlags.generations; g++) {
@@ -155,9 +156,9 @@ initStorage( void )
   /* The oldest generation has one step. */
   if (RtsFlags.GcFlags.compact || RtsFlags.GcFlags.sweep) {
       if (RtsFlags.GcFlags.generations == 1) {
-	  errorBelch("WARNING: compact/sweep is incompatible with -G1; disabled");
+          errorBelch("WARNING: compact/sweep is incompatible with -G1; disabled");
       } else {
-	  oldest_gen->mark = 1;
+          oldest_gen->mark = 1;
           if (RtsFlags.GcFlags.compact)
               oldest_gen->compact = 1;
       }
@@ -186,6 +187,13 @@ initStorage( void )
   IF_DEBUG(gc, statDescribeGens());
 
   RELEASE_SM_LOCK;
+
+  traceEventHeapInfo(CAPSET_HEAP_DEFAULT,
+                     RtsFlags.GcFlags.generations,
+                     RtsFlags.GcFlags.maxHeapSize * BLOCK_SIZE_W * sizeof(W_),
+                     RtsFlags.GcFlags.minAllocAreaSize * BLOCK_SIZE_W * sizeof(W_),
+                     MBLOCK_SIZE_W * sizeof(W_),
+                     BLOCK_SIZE_W  * sizeof(W_));
 }
 
 void storageAddCapabilities (nat from, nat to)
@@ -197,7 +205,7 @@ void storageAddCapabilities (nat from, nat to)
                                     "storageAddCapabilities");
     } else {
         nurseries = stgMallocBytes(to * sizeof(struct nursery_),
-                                    "storageAddCapabilities");
+                                   "storageAddCapabilities");
     }
 
     // we've moved the nurseries, so we have to update the rNursery
@@ -228,7 +236,8 @@ void storageAddCapabilities (nat from, nat to)
 void
 exitStorage (void)
 {
-    stat_exit(calcAllocated(rtsTrue));
+    W_ allocated = updateNurseriesStats();
+    stat_exit(allocated);
 }
 
 void
@@ -271,7 +280,7 @@ freeStorage (rtsBool free_heap)
 
       - it puts the CAF on the oldest generation's mutable list.
         This is so that we treat the CAF as a root when collecting
-	younger generations.
+        younger generations.
 
    ------------------
    Note [atomic CAF entry]
@@ -417,10 +426,10 @@ newDynCAF (StgRegTable *reg STG_UNUSED, StgClosure *caf, StgClosure *bh)
    -------------------------------------------------------------------------- */
 
 static bdescr *
-allocNursery (bdescr *tail, nat blocks)
+allocNursery (bdescr *tail, W_ blocks)
 {
     bdescr *bd = NULL;
-    nat i, n;
+    W_ i, n;
 
     // We allocate the nursery as a single contiguous block and then
     // divide it into single blocks manually.  This way we guarantee
@@ -429,10 +438,14 @@ allocNursery (bdescr *tail, nat blocks)
     // tiny optimisation (~0.5%), but it's free.
 
     while (blocks > 0) {
-        n = stg_min(blocks, BLOCKS_PER_MBLOCK);
+        n = stg_min(BLOCKS_PER_MBLOCK, blocks);
+        // allocLargeChunk will prefer large chunks, but will pick up
+        // small chunks if there are any available.  We must allow
+        // single blocks here to avoid fragmentation (#7257)
+        bd = allocLargeChunk(1, n);
+        n = bd->blocks;
         blocks -= n;
 
-        bd = allocGroup(n);
         for (i = 0; i < n; i++) {
             initBdescr(&bd[i], g0, g0);
 
@@ -470,7 +483,7 @@ assignNurseriesToCapabilities (nat from, nat to)
 
     for (i = from; i < to; i++) {
         capabilities[i].r.rCurrentNursery = nurseries[i].blocks;
-	capabilities[i].r.rCurrentAlloc   = NULL;
+        capabilities[i].r.rCurrentAlloc   = NULL;
     }
 }
 
@@ -482,27 +495,25 @@ allocNurseries (nat from, nat to)
     for (i = from; i < to; i++) {
         nurseries[i].blocks =
             allocNursery(NULL, RtsFlags.GcFlags.minAllocAreaSize);
-	nurseries[i].n_blocks =
+        nurseries[i].n_blocks =
             RtsFlags.GcFlags.minAllocAreaSize;
     }
     assignNurseriesToCapabilities(from, to);
 }
       
-lnat // words allocated
-clearNurseries (void)
+W_
+clearNursery (Capability *cap)
 {
-    lnat allocated = 0;
-    nat i;
     bdescr *bd;
+    W_ allocated = 0;
 
-    for (i = 0; i < n_capabilities; i++) {
-	for (bd = nurseries[i].blocks; bd; bd = bd->link) {
-            allocated += (lnat)(bd->free - bd->start);
-            bd->free = bd->start;
-	    ASSERT(bd->gen_no == 0);
-	    ASSERT(bd->gen == g0);
-	    IF_DEBUG(sanity,memset(bd->start, 0xaa, BLOCK_SIZE));
-	}
+    for (bd = nurseries[cap->no].blocks; bd; bd = bd->link) {
+        allocated            += (W_)(bd->free - bd->start);
+        cap->total_allocated += (W_)(bd->free - bd->start);
+        bd->free = bd->start;
+        ASSERT(bd->gen_no == 0);
+        ASSERT(bd->gen == g0);
+        IF_DEBUG(sanity,memset(bd->start, 0xaa, BLOCK_SIZE));
     }
 
     return allocated;
@@ -514,51 +525,51 @@ resetNurseries (void)
     assignNurseriesToCapabilities(0, n_capabilities);
 }
 
-lnat
+W_
 countNurseryBlocks (void)
 {
     nat i;
-    lnat blocks = 0;
+    W_ blocks = 0;
 
     for (i = 0; i < n_capabilities; i++) {
-	blocks += nurseries[i].n_blocks;
+        blocks += nurseries[i].n_blocks;
     }
     return blocks;
 }
 
 static void
-resizeNursery ( nursery *nursery, nat blocks )
+resizeNursery (nursery *nursery, W_ blocks)
 {
   bdescr *bd;
-  nat nursery_blocks;
+  W_ nursery_blocks;
 
   nursery_blocks = nursery->n_blocks;
   if (nursery_blocks == blocks) return;
 
   if (nursery_blocks < blocks) {
       debugTrace(DEBUG_gc, "increasing size of nursery to %d blocks", 
-		 blocks);
+                 blocks);
     nursery->blocks = allocNursery(nursery->blocks, blocks-nursery_blocks);
   } 
   else {
     bdescr *next_bd;
     
     debugTrace(DEBUG_gc, "decreasing size of nursery to %d blocks", 
-	       blocks);
+               blocks);
 
     bd = nursery->blocks;
     while (nursery_blocks > blocks) {
-	next_bd = bd->link;
-	next_bd->u.back = NULL;
-	nursery_blocks -= bd->blocks; // might be a large block
-	freeGroup(bd);
-	bd = next_bd;
+        next_bd = bd->link;
+        next_bd->u.back = NULL;
+        nursery_blocks -= bd->blocks; // might be a large block
+        freeGroup(bd);
+        bd = next_bd;
     }
     nursery->blocks = bd;
     // might have gone just under, by freeing a large block, so make
     // up the difference.
     if (nursery_blocks < blocks) {
-	nursery->blocks = allocNursery(nursery->blocks, blocks-nursery_blocks);
+        nursery->blocks = allocNursery(nursery->blocks, blocks-nursery_blocks);
     }
   }
   
@@ -570,11 +581,11 @@ resizeNursery ( nursery *nursery, nat blocks )
 // Resize each of the nurseries to the specified size.
 //
 void
-resizeNurseriesFixed (nat blocks)
+resizeNurseriesFixed (W_ blocks)
 {
     nat i;
     for (i = 0; i < n_capabilities; i++) {
-	resizeNursery(&nurseries[i], blocks);
+        resizeNursery(&nurseries[i], blocks);
     }
 }
 
@@ -582,7 +593,7 @@ resizeNurseriesFixed (nat blocks)
 // Resize the nurseries to the total specified size.
 //
 void
-resizeNurseries (nat blocks)
+resizeNurseries (W_ blocks)
 {
     // If there are multiple nurseries, then we just divide the number
     // of available blocks between them.
@@ -619,7 +630,7 @@ move_STACK (StgStack *src, StgStack *dest)
    -------------------------------------------------------------------------- */
 
 StgPtr
-allocate (Capability *cap, lnat n)
+allocate (Capability *cap, W_ n)
 {
     bdescr *bd;
     StgPtr p;
@@ -628,12 +639,15 @@ allocate (Capability *cap, lnat n)
     CCS_ALLOC(cap->r.rCCCS,n);
     
     if (n >= LARGE_OBJECT_THRESHOLD/sizeof(W_)) {
-	lnat req_blocks =  (lnat)BLOCK_ROUND_UP(n*sizeof(W_)) / BLOCK_SIZE;
+        W_ req_blocks =  (W_)BLOCK_ROUND_UP(n*sizeof(W_)) / BLOCK_SIZE;
 
         // Attempting to allocate an object larger than maxHeapSize
         // should definitely be disallowed.  (bug #1791)
-        if (RtsFlags.GcFlags.maxHeapSize > 0 && 
-            req_blocks >= RtsFlags.GcFlags.maxHeapSize) {
+        if ((RtsFlags.GcFlags.maxHeapSize > 0 &&
+             req_blocks >= RtsFlags.GcFlags.maxHeapSize) ||
+            req_blocks >= HS_INT32_MAX)   // avoid overflow when
+                                          // calling allocGroup() below
+        {
             heapOverflow();
             // heapOverflow() doesn't exit (see #2592), but we aren't
             // in a position to do a clean shutdown here: we
@@ -641,19 +655,20 @@ allocate (Capability *cap, lnat n)
             // Allocating the memory would be bad, because the user
             // has requested that we not exceed maxHeapSize, so we
             // just exit.
-	    stg_exit(EXIT_HEAPOVERFLOW);
+            stg_exit(EXIT_HEAPOVERFLOW);
         }
 
         ACQUIRE_SM_LOCK
-	bd = allocGroup(req_blocks);
-	dbl_link_onto(bd, &g0->large_objects);
-	g0->n_large_blocks += bd->blocks; // might be larger than req_blocks
+        bd = allocGroup(req_blocks);
+        dbl_link_onto(bd, &g0->large_objects);
+        g0->n_large_blocks += bd->blocks; // might be larger than req_blocks
         g0->n_new_large_words += n;
         RELEASE_SM_LOCK;
         initBdescr(bd, g0, g0);
-	bd->flags = BF_LARGE;
-	bd->free = bd->start + n;
-	return bd->start;
+        bd->flags = BF_LARGE;
+        bd->free = bd->start + n;
+        cap->total_allocated += n;
+        return bd->start;
     }
 
     /* small allocation (<LARGE_OBJECT_THRESHOLD) */
@@ -722,7 +737,7 @@ allocate (Capability *cap, lnat n)
    ------------------------------------------------------------------------- */
 
 StgPtr
-allocatePinned (Capability *cap, lnat n)
+allocatePinned (Capability *cap, W_ n)
 {
     StgPtr p;
     bdescr *bd;
@@ -730,7 +745,7 @@ allocatePinned (Capability *cap, lnat n)
     // If the request is for a large object, then allocate()
     // will give us a pinned object anyway.
     if (n >= LARGE_OBJECT_THRESHOLD/sizeof(W_)) {
-	p = allocate(cap, n);
+        p = allocate(cap, n);
         Bdescr(p)->flags |= BF_PINNED;
         return p;
     }
@@ -741,8 +756,55 @@ allocatePinned (Capability *cap, lnat n)
     bd = cap->pinned_object_block;
     
     // If we don't have a block of pinned objects yet, or the current
-    // one isn't large enough to hold the new object, allocate a new one.
+    // one isn't large enough to hold the new object, get a new one.
     if (bd == NULL || (bd->free + n) > (bd->start + BLOCK_SIZE_W)) {
+
+        // stash the old block on cap->pinned_object_blocks.  On the
+        // next GC cycle these objects will be moved to
+        // g0->large_objects.
+        if (bd != NULL) {
+            dbl_link_onto(bd, &cap->pinned_object_blocks);
+            cap->total_allocated += bd->free - bd->start;
+        }
+
+        // We need to find another block.  We could just allocate one,
+        // but that means taking a global lock and we really want to
+        // avoid that (benchmarks that allocate a lot of pinned
+        // objects scale really badly if we do this).
+        //
+        // So first, we try taking the next block from the nursery, in
+        // the same way as allocate(), but note that we can only take
+        // an *empty* block, because we're about to mark it as
+        // BF_PINNED | BF_LARGE.
+        bd = cap->r.rCurrentNursery->link;
+        if (bd == NULL || bd->free != bd->start) { // must be empty!
+            // The nursery is empty, or the next block is non-empty:
+            // allocate a fresh block (we can't fail here).
+
+            // XXX in the case when the next nursery block is
+            // non-empty we aren't exerting any pressure to GC soon,
+            // so if this case ever happens then we could in theory
+            // keep allocating for ever without calling the GC. We
+            // can't bump g0->n_new_large_words because that will be
+            // counted towards allocation, and we're already counting
+            // our pinned obects as allocation in
+            // collect_pinned_object_blocks in the GC.
+            ACQUIRE_SM_LOCK;
+            bd = allocBlock();
+            RELEASE_SM_LOCK;
+            initBdescr(bd, g0, g0);
+        } else {
+            // we have a block in the nursery: steal it
+            cap->r.rCurrentNursery->link = bd->link;
+            if (bd->link != NULL) {
+                bd->link->u.back = cap->r.rCurrentNursery;
+            }
+            cap->r.rNursery->n_blocks -= bd->blocks;
+        }
+
+        cap->pinned_object_block = bd;
+        bd->flags  = BF_PINNED | BF_LARGE | BF_EVACUATED;
+
         // The pinned_object_block remains attached to the capability
         // until it is full, even if a GC occurs.  We want this
         // behaviour because otherwise the unallocated portion of the
@@ -756,17 +818,6 @@ allocatePinned (Capability *cap, lnat n)
         // the next GC the BF_EVACUATED flag will be cleared, and the
         // block will be promoted as usual (if anything in it is
         // live).
-        ACQUIRE_SM_LOCK;
-        if (bd != NULL) {
-            dbl_link_onto(bd, &g0->large_objects);
-            g0->n_large_blocks++;
-            g0->n_new_large_words += bd->free - bd->start;
-        }
-        cap->pinned_object_block = bd = allocBlock();
-        RELEASE_SM_LOCK;
-        initBdescr(bd, g0, g0);
-        bd->flags  = BF_PINNED | BF_LARGE | BF_EVACUATED;
-	bd->free   = bd->start;
     }
 
     p = bd->free;
@@ -789,7 +840,7 @@ dirty_MUT_VAR(StgRegTable *reg, StgClosure *p)
 {
     Capability *cap = regTableToCapability(reg);
     if (p->header.info == &stg_MUT_VAR_CLEAN_info) {
-	p->header.info = &stg_MUT_VAR_DIRTY_info;
+        p->header.info = &stg_MUT_VAR_DIRTY_info;
         recordClosureMutated(cap,p);
     }
 }
@@ -857,38 +908,40 @@ dirty_MVAR(StgRegTable *reg, StgClosure *p)
  * -------------------------------------------------------------------------- */
 
 /* -----------------------------------------------------------------------------
- * calcAllocated()
+ * updateNurseriesStats()
  *
- * Approximate how much we've allocated: number of blocks in the
- * nursery + blocks allocated via allocate() - unused nusery blocks.
- * This leaves a little slop at the end of each block.
+ * Update the per-cap total_allocated numbers with an approximation of
+ * the amount of memory used in each cap's nursery. Also return the
+ * total across all caps.
+ * 
+ * Since this update is also performed by clearNurseries() then we only
+ * need this function for the final stats when the RTS is shutting down.
  * -------------------------------------------------------------------------- */
 
-lnat
-calcAllocated (rtsBool include_nurseries)
+W_
+updateNurseriesStats (void)
 {
-  nat allocated = 0;
-  nat i;
+    W_ allocated = 0;
+    nat i;
 
-  // When called from GC.c, we already have the allocation count for
-  // the nursery from resetNurseries(), so we don't need to walk
-  // through these block lists again.
-  if (include_nurseries)
-  {
-      for (i = 0; i < n_capabilities; i++) {
-          allocated += countOccupied(nurseries[i].blocks);
-      }
-  }
+    for (i = 0; i < n_capabilities; i++) {
+        int cap_allocated = countOccupied(nurseries[i].blocks);
+        capabilities[i].total_allocated += cap_allocated;
+        allocated                       += cap_allocated;
+    }
 
-  // add in sizes of new large and pinned objects
-  allocated += g0->n_new_large_words;
+    return allocated;
+}
 
-  return allocated;
-}  
-
-lnat countOccupied (bdescr *bd)
+W_
+countLargeAllocated (void)
 {
-    lnat words;
+    return g0->n_new_large_words;
+}
+
+W_ countOccupied (bdescr *bd)
+{
+    W_ words;
 
     words = 0;
     for (; bd != NULL; bd = bd->link) {
@@ -898,19 +951,19 @@ lnat countOccupied (bdescr *bd)
     return words;
 }
 
-lnat genLiveWords (generation *gen)
+W_ genLiveWords (generation *gen)
 {
-    return gen->n_words + countOccupied(gen->large_objects);
+    return gen->n_words + gen->n_large_words;
 }
 
-lnat genLiveBlocks (generation *gen)
+W_ genLiveBlocks (generation *gen)
 {
     return gen->n_blocks + gen->n_large_blocks;
 }
 
-lnat gcThreadLiveWords (nat i, nat g)
+W_ gcThreadLiveWords (nat i, nat g)
 {
-    lnat words;
+    W_ words;
 
     words   = countOccupied(gc_threads[i]->gens[g].todo_bd);
     words  += countOccupied(gc_threads[i]->gens[g].part_list);
@@ -919,9 +972,9 @@ lnat gcThreadLiveWords (nat i, nat g)
     return words;
 }
 
-lnat gcThreadLiveBlocks (nat i, nat g)
+W_ gcThreadLiveBlocks (nat i, nat g)
 {
-    lnat blocks;
+    W_ blocks;
 
     blocks  = countBlocks(gc_threads[i]->gens[g].todo_bd);
     blocks += gc_threads[i]->gens[g].n_part_blocks;
@@ -932,10 +985,10 @@ lnat gcThreadLiveBlocks (nat i, nat g)
 
 // Return an accurate count of the live data in the heap, excluding
 // generation 0.
-lnat calcLiveWords (void)
+W_ calcLiveWords (void)
 {
     nat g;
-    lnat live;
+    W_ live;
 
     live = 0;
     for (g = 0; g < RtsFlags.GcFlags.generations; g++) {
@@ -944,10 +997,10 @@ lnat calcLiveWords (void)
     return live;
 }
 
-lnat calcLiveBlocks (void)
+W_ calcLiveBlocks (void)
 {
     nat g;
-    lnat live;
+    W_ live;
 
     live = 0;
     for (g = 0; g < RtsFlags.GcFlags.generations; g++) {
@@ -963,10 +1016,10 @@ lnat calcLiveBlocks (void)
  * that will be collected next time will therefore need twice as many
  * blocks since all the data will be copied.
  */
-extern lnat 
+extern W_ 
 calcNeeded(void)
 {
-    lnat needed = 0;
+    W_ needed = 0;
     nat g;
     generation *gen;
     
@@ -1022,7 +1075,7 @@ calcNeeded(void)
 // because it knows how to work around the restrictions put in place
 // by SELinux.
 
-void *allocateExec (nat bytes, void **exec_ret)
+void *allocateExec (W_ bytes, void **exec_ret)
 {
     void **ret, **exec;
     ACQUIRE_SM_LOCK;
@@ -1046,10 +1099,10 @@ void freeExec (void *addr)
 
 #else
 
-void *allocateExec (nat bytes, void **exec_ret)
+void *allocateExec (W_ bytes, void **exec_ret)
 {
     void *ret;
-    nat n;
+    W_ n;
 
     ACQUIRE_SM_LOCK;
 
@@ -1057,24 +1110,24 @@ void *allocateExec (nat bytes, void **exec_ret)
     n  = (bytes + sizeof(W_) + 1) / sizeof(W_);
 
     if (n+1 > BLOCK_SIZE_W) {
-	barf("allocateExec: can't handle large objects");
+        barf("allocateExec: can't handle large objects");
     }
 
     if (exec_block == NULL || 
-	exec_block->free + n + 1 > exec_block->start + BLOCK_SIZE_W) {
-	bdescr *bd;
-	lnat pagesize = getPageSize();
-	bd = allocGroup(stg_max(1, pagesize / BLOCK_SIZE));
-	debugTrace(DEBUG_gc, "allocate exec block %p", bd->start);
-	bd->gen_no = 0;
-	bd->flags = BF_EXEC;
-	bd->link = exec_block;
-	if (exec_block != NULL) {
-	    exec_block->u.back = bd;
-	}
-	bd->u.back = NULL;
-	setExecutable(bd->start, bd->blocks * BLOCK_SIZE, rtsTrue);
-	exec_block = bd;
+        exec_block->free + n + 1 > exec_block->start + BLOCK_SIZE_W) {
+        bdescr *bd;
+        W_ pagesize = getPageSize();
+        bd = allocGroup(stg_max(1, pagesize / BLOCK_SIZE));
+        debugTrace(DEBUG_gc, "allocate exec block %p", bd->start);
+        bd->gen_no = 0;
+        bd->flags = BF_EXEC;
+        bd->link = exec_block;
+        if (exec_block != NULL) {
+            exec_block->u.back = bd;
+        }
+        bd->u.back = NULL;
+        setExecutable(bd->start, bd->blocks * BLOCK_SIZE, rtsTrue);
+        exec_block = bd;
     }
     *(exec_block->free) = n;  // store the size of this chunk
     exec_block->gen_no += n;  // gen_no stores the number of words allocated
@@ -1092,11 +1145,11 @@ void freeExec (void *addr)
     bdescr *bd = Bdescr((StgPtr)p);
 
     if ((bd->flags & BF_EXEC) == 0) {
-	barf("freeExec: not executable");
+        barf("freeExec: not executable");
     }
 
     if (*(StgPtr)p == 0) {
-	barf("freeExec: already free?");
+        barf("freeExec: already free?");
     }
 
     ACQUIRE_SM_LOCK;
@@ -1125,10 +1178,10 @@ void freeExec (void *addr)
 #ifdef DEBUG
 
 // handy function for use in gdb, because Bdescr() is inlined.
-extern bdescr *_bdescr( StgPtr p );
+extern bdescr *_bdescr (StgPtr p);
 
 bdescr *
-_bdescr( StgPtr p )
+_bdescr (StgPtr p)
 {
     return Bdescr(p);
 }
