@@ -152,8 +152,12 @@ ioManagerStart (void)
     Capability *cap;
     if (io_manager_event == INVALID_HANDLE_VALUE) {
 	cap = rts_lock();
-#if defined(mingw32_HOST_OS) && defined(__PIC__)
+#if defined(COMPILING_WINDOWS_DLL)
+#  if defined(x86_64_HOST_ARCH)
+        rts_evalIO(&cap,__imp_base_GHCziConcziIO_ensureIOManagerIsRunning_closure,NULL);
+#  else
         rts_evalIO(&cap,_imp__base_GHCziConcziIO_ensureIOManagerIsRunning_closure,NULL);
+#  endif
 #else
         rts_evalIO(&cap,&base_GHCziConcziIO_ensureIOManagerIsRunning_closure,NULL);
 #endif
