@@ -1,5 +1,5 @@
-{-# OPTIONS_HADDOCK hide #-}
 {-# LANGUAGE DeriveDataTypeable, DeriveFunctor #-}
+{-# OPTIONS_GHC -fno-warn-orphans #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Haddock.Types
@@ -22,6 +22,7 @@ module Haddock.Types (
 
 import Control.Exception
 import Control.Arrow
+import Control.DeepSeq
 import Data.Typeable
 import Data.Map (Map)
 import Data.Maybe
@@ -59,10 +60,10 @@ type DocPaths      = (FilePath, Maybe FilePath) -- paths to HTML and sources
 data Interface = Interface
   {
     -- | The module behind this interface.
-    ifaceMod             :: Module
+    ifaceMod             :: !Module
 
     -- | Original file name of the module.
-  , ifaceOrigFilename    :: FilePath
+  , ifaceOrigFilename    :: !FilePath
 
     -- | Textual information about the module.
   , ifaceInfo            :: !(HaddockModInfo Name)
@@ -71,7 +72,7 @@ data Interface = Interface
   , ifaceDoc             :: !(Documentation Name)
 
     -- | Documentation header with cross-reference information.
-  , ifaceRnDoc           :: Documentation DocName
+  , ifaceRnDoc           :: !(Documentation DocName)
 
     -- | Haddock options for this module (prune, ignore-exports, etc).
   , ifaceOptions         :: ![DocOption]
@@ -79,22 +80,22 @@ data Interface = Interface
     -- | Declarations originating from the module. Excludes declarations without
     -- names (instances and stand-alone documentation comments). Includes
     -- names of subordinate declarations mapped to their parent declarations.
-  , ifaceDeclMap         :: Map Name [LHsDecl Name]
+  , ifaceDeclMap         :: !(Map Name [LHsDecl Name])
 
     -- | Documentation of declarations originating from the module (including
     -- subordinates).
-  , ifaceDocMap          :: DocMap Name
-  , ifaceArgMap          :: ArgMap Name
+  , ifaceDocMap          :: !(DocMap Name)
+  , ifaceArgMap          :: !(ArgMap Name)
 
     -- | Documentation of declarations originating from the module (including
     -- subordinates).
-  , ifaceRnDocMap        :: DocMap DocName
-  , ifaceRnArgMap        :: ArgMap DocName
+  , ifaceRnDocMap        :: !(DocMap DocName)
+  , ifaceRnArgMap        :: !(ArgMap DocName)
 
-  , ifaceSubMap          :: Map Name [Name]
+  , ifaceSubMap          :: !(Map Name [Name])
 
   , ifaceExportItems     :: ![ExportItem Name]
-  , ifaceRnExportItems   :: [ExportItem DocName]
+  , ifaceRnExportItems   :: ![ExportItem DocName]
 
     -- | All names exported by the module.
   , ifaceExports         :: ![Name]
@@ -105,16 +106,21 @@ data Interface = Interface
   , ifaceVisibleExports  :: ![Name]
 
     -- | Aliases of module imports as in @import A.B.C as C@.
-  , ifaceModuleAliases   :: AliasMap
+  , ifaceModuleAliases   :: !AliasMap
 
     -- | Instances exported by the module.
-  , ifaceInstances       :: ![Instance]
+  , ifaceInstances       :: ![ClsInst]
 
     -- | The number of haddockable and haddocked items in the module, as a
     -- tuple. Haddockable items are the exports and the module itself.
-  , ifaceHaddockCoverage :: (Int,Int)
+  , ifaceHaddockCoverage :: !(Int, Int)
+
+    -- | Warnings for things defined in this module.
+  , ifaceWarningMap :: !WarningMap
   }
 
+type WarningMap = DocMap Name
+
 
 -- | A subset of the fields of 'Interface' that we store in the interface
 -- files.
@@ -172,51 +178,51 @@ data ExportItem name
   = ExportDecl
       {
         -- | A declaration.
-        expItemDecl :: LHsDecl name
+        expItemDecl :: !(LHsDecl name)
 
         -- | Maybe a doc comment, and possibly docs for arguments (if this
         -- decl is a function or type-synonym).
-      , expItemMbDoc :: DocForDecl name
+      , expItemMbDoc :: !(DocForDecl name)
 
         -- | Subordinate names, possibly with documentation.
-      , expItemSubDocs :: [(name, DocForDecl name)]
+      , expItemSubDocs :: ![(name, DocForDecl name)]
 
         -- | Instances relevant to this declaration, possibly with
         -- documentation.
-      , expItemInstances :: [DocInstance name]
+      , expItemInstances :: ![DocInstance name]
       }
 
   -- | An exported entity for which we have no documentation (perhaps because it
   -- resides in another package).
   | ExportNoDecl
-      { expItemName :: name
+      { expItemName :: !name
 
         -- | Subordinate names.
-      , expItemSubs :: [name]
+      , expItemSubs :: ![name]
       }
 
   -- | A section heading. 
   | ExportGroup
       {
         -- | Section level (1, 2, 3, ...).
-        expItemSectionLevel :: Int
+        expItemSectionLevel :: !Int
 
         -- | Section id (for hyperlinks).
-      , expItemSectionId :: String
+      , expItemSectionId :: !String
 
         -- | Section heading text.
-      , expItemSectionText :: Doc name
+      , expItemSectionText :: !(Doc name)
       }
 
   -- | Some documentation.
-  | ExportDoc (Doc name)
+  | ExportDoc !(Doc name)
 
   -- | A cross-reference to another module.
-  | ExportModule Module
+  | ExportModule !Module
 
 data Documentation name = Documentation
   { documentationDoc :: Maybe (Doc name)
-  , documentationWarning :: Maybe (Doc name)
+  , documentationWarning :: !(Maybe (Doc name))
   } deriving Functor
 
 
@@ -303,9 +309,10 @@ data Doc id
   | DocOrderedList [Doc id]
   | DocDefList [(Doc id, Doc id)]
   | DocCodeBlock (Doc id)
-  | DocURL String
+  | DocHyperlink Hyperlink
   | DocPic String
   | DocAName String
+  | DocProperty String
   | DocExamples [Example]
   deriving (Functor)
 
@@ -315,12 +322,54 @@ instance Monoid (Doc id) where
   mappend = DocAppend
 
 
+instance NFData a => NFData (Doc a) where
+  rnf doc = case doc of
+    DocEmpty                  -> ()
+    DocAppend a b             -> a `deepseq` b `deepseq` ()
+    DocString a               -> a `deepseq` ()
+    DocParagraph a            -> a `deepseq` ()
+    DocIdentifier a           -> a `deepseq` ()
+    DocIdentifierUnchecked a  -> a `deepseq` ()
+    DocModule a               -> a `deepseq` ()
+    DocWarning a              -> a `deepseq` ()
+    DocEmphasis a             -> a `deepseq` ()
+    DocMonospaced a           -> a `deepseq` ()
+    DocUnorderedList a        -> a `deepseq` ()
+    DocOrderedList a          -> a `deepseq` ()
+    DocDefList a              -> a `deepseq` ()
+    DocCodeBlock a            -> a `deepseq` ()
+    DocHyperlink a            -> a `deepseq` ()
+    DocPic a                  -> a `deepseq` ()
+    DocAName a                -> a `deepseq` ()
+    DocProperty a             -> a `deepseq` ()
+    DocExamples a             -> a `deepseq` ()
+
+
+instance NFData Name
+instance NFData OccName
+instance NFData ModuleName
+
+
+data Hyperlink = Hyperlink
+  { hyperlinkUrl   :: String
+  , hyperlinkLabel :: Maybe String
+  } deriving (Eq, Show)
+
+
+instance NFData Hyperlink where
+  rnf (Hyperlink a b) = a `deepseq` b `deepseq` ()
+
+
 data Example = Example
   { exampleExpression :: String
   , exampleResult     :: [String]
   } deriving (Eq, Show)
 
 
+instance NFData Example where
+  rnf (Example a b) = a `deepseq` b `deepseq` ()
+
+
 exampleToString :: Example -> String
 exampleToString (Example expression result) =
     ">>> " ++ expression ++ "\n" ++  unlines result
@@ -341,19 +390,20 @@ data DocMarkup id a = Markup
   , markupOrderedList          :: [a] -> a
   , markupDefList              :: [(a,a)] -> a
   , markupCodeBlock            :: a -> a
-  , markupURL                  :: String -> a
+  , markupHyperlink            :: Hyperlink -> a
   , markupAName                :: String -> a
   , markupPic                  :: String -> a
+  , markupProperty             :: String -> a
   , markupExample              :: [Example] -> a
   }
 
 
 data HaddockModInfo name = HaddockModInfo
-  { hmi_description :: Maybe (Doc name)
-  , hmi_portability :: Maybe String
-  , hmi_stability   :: Maybe String
-  , hmi_maintainer  :: Maybe String
-  , hmi_safety      :: Maybe String
+  { hmi_description :: (Maybe (Doc name))
+  , hmi_portability :: (Maybe String)
+  , hmi_stability   :: (Maybe String)
+  , hmi_maintainer  :: (Maybe String)
+  , hmi_safety      :: (Maybe String)
   }
 
 
