@@ -25,6 +25,7 @@ import Foreign.C.String
 import System.Directory         ( doesFileExist, findExecutable )
 import System.Environment       ( getProgName, getArgs )
 import System.Exit              ( ExitCode(..), exitWith )
+import System.FilePath          ( normalise, splitFileName, splitExtension )
 import System.IO
 
 #ifdef BUILD_NHC
@@ -40,6 +41,16 @@ import DirectCodegen
 import Flags
 import HSCParser
 
+#ifdef mingw32_HOST_OS
+# if defined(i386_HOST_ARCH)
+#  define WINDOWS_CCONV stdcall
+# elif defined(x86_64_HOST_ARCH)
+#  define WINDOWS_CCONV ccall
+# else
+#  error Unknown mingw32 arch
+# endif
+#endif
+
 #ifdef BUILD_NHC
 getDataFileName s = do here <- getCurrentDirectory
                        return (here++"/"++s)
@@ -107,14 +118,14 @@ processFiles configM files usage = do
                             then return (dir++base++"_out.hs", dir, base)
                             else return (dir++base++".hs",     dir, base)
                    where
-                    (dir,  file) = splitName name
-                    (base, ext)  = splitExt  file
+                    (dir,  file) = splitFileName  name
+                    (base, ext)  = splitExtension file
              [f] -> let
-                 (dir,  file) = splitName f
-                 (base, _)    = splitExt file
+                 (dir,  file) = splitFileName  f
+                 (base, _)    = splitExtension file
                  in return (f, dir, base)
              _ -> onlyOne "output file"
-        let file_name = dosifyPath name
+        let file_name = normalise name
         toks <- parseFile file_name
         outputter config outName outDir outBase file_name toks)
 
@@ -170,16 +181,16 @@ findCompiler mb_libdir config
                       Nothing ->
                           die ("Can't find "++default_compiler++"\n")
                       Just path -> return path
-          -- if this hsc2hs is part of a GHC installation on
-          -- Windows, then we should use the mingw gcc that
-          -- comes with GHC (#3929)
-          case mb_libdir of
-              Nothing -> search_path
-              Just d  ->
-                  do let inplaceGcc = d ++ "/../mingw/bin/gcc.exe"
-                     b <- doesFileExist inplaceGcc
-                     if b then return inplaceGcc
-                          else search_path
+              -- if this hsc2hs is part of a GHC installation on
+              -- Windows, then we should use the mingw gcc that
+              -- comes with GHC (#3929)
+              inplaceGccs = case mb_libdir of
+                            Nothing -> []
+                            Just d  -> [d ++ "/../mingw/bin/gcc.exe"]
+              search [] = search_path
+              search (x : xs) = do b <- doesFileExist x
+                                   if b then return x else search xs
+          search inplaceGccs
 
 parseFile :: String -> IO [Token]
 parseFile name
@@ -208,7 +219,7 @@ getExecDir :: String -> IO (Maybe String)
 getExecDir cmd =
     getExecPath >>= maybe (return Nothing) removeCmdSuffix
     where initN n = reverse . drop n . reverse
-          removeCmdSuffix = return . Just . initN (length cmd) . unDosifyPath
+          removeCmdSuffix = return . Just . initN (length cmd) . normalise
 
 getExecPath :: IO (Maybe String)
 #if defined(mingw32_HOST_OS)
@@ -221,7 +232,7 @@ getExecPath = try_size 2048 -- plenty, PATH_MAX is 512 under Win32.
           _ | ret < size -> fmap Just $ peekCWString buf
             | otherwise  -> try_size (size * 2)
 
-foreign import stdcall unsafe "windows.h GetModuleFileNameW"
+foreign import WINDOWS_CCONV unsafe "windows.h GetModuleFileNameW"
   c_GetModuleFileName :: Ptr () -> CWString -> Word32 -> IO Word32
 #else
 getExecPath = return Nothing
